local action_end_date = 0
local num_tick = 0

function on_timer (tick)
	update_timer()
end

function update_timer()
	if dialog then
		local date_time_label = dialog:find_child('date_time', true, false)
		if date_time_label then
			date_time_label.time = controller.remaining_time
		end
		
		if controller.remaining_time <= 0 then
			dialog:end_dialog()
		end
	end
end

function on_init_dialog()

	dialog:load_content('build_in_time_dlg')
	dialog.center_dialog = true

	dialog.theme_flags = 2
	
	dialog.caption = controller.title
	
	local build_table = dialog:find_child('build_table', true, false)
	if build_table then
		build_table:update_content()
	end
	
	local description_label = dialog:find_child('description', true, false) 
	if description_label then
		description_label.caption = controller.description
	end
		
	local object_info = framework:object_factory():get_object_info_by_type_id(controller.reward_type_id)
	if object_info then				
		
		local reward_caption_label = dialog:find_child('reward_caption_label', true, false)
		if reward_caption_label then
			reward_caption_label.caption = framework:strings():load("*CityBuildInTimeRewardCaption", "REWARD!")
			reward_caption_label.text_adjust_font_size = true
		end
		
		local reward_item_label = dialog:find_child('reward_item_label', true, false)
		if reward_item_label then
			reward_item_label.caption = object_info.description
			reward_item_label.text_adjust_font_size = true
		end
	
		local reward_image = dialog:find_child('reward_image', true, false)
		if reward_image then
			reward_image.texture_name = object_info.shop_image_path
		end
		
		local population = dialog:find_child('population_label', true, false)
		if population then
			population.caption = "+"..object_info.max_population_increase
			population.text_adjust_font_size = true
		end
		
		local coins = dialog:find_child('coins_label', true, false)
		if coins then
			coins.caption = "+"..object_info.extra_coins
			coins.text_adjust_font_size = true
		end
		
		local water = dialog:find_child('water_label', true, false)
		if water then
			water.caption = object_info.water_consume
			water.text_adjust_font_size = true
		end
		
		local power = dialog:find_child('power_label', true, false)
		if power then
			power.caption = object_info.power_consume
			power.text_adjust_font_size = true
		end
		
		local exp = dialog:find_child('exp_label', true, false)
		if exp then
			exp.caption = object_info.exp
			exp.text_adjust_font_size = true
		end
		
		local time_label = dialog:find_child('time_label', true, false)
		if time_label then
			time_label.caption = framework:formats():format_time_in_seconds(object_info.abandoned_length)
			time_label.text_adjust_font_size = true
		end
	end
	
	
	if controller.completed == false then
		local proceed_button_label = dialog:find_child("proceed_button_label", true, false)
		if proceed_button_label then
			proceed_button_label.caption = framework:strings():load("*CityBuildInTimeProceed", "Proceed")
		end
	else
		local proceed_button_label = dialog:find_child("proceed_button_label", true, false)
		if proceed_button_label then
			proceed_button_label.caption = framework:strings():load("*CityActionTakePresent", "Take reward")
		end		
	end
	
end

function execute_goal(idx)
	controller:goto_button_action(controller:get_event_progress()[idx+1])
end

build_table_delegate = {
	create_cell = function (  self, idx, rect )
		
		local w = self.table:create_child(rect)
		                
        framework:json_ui_manager():build_from_resource("build_in_time_table_cell",w)
        w.rect = rect
       	
       	local cell = w:find_child("table_cell", true, false)
       	if cell then
       		cell.tag = cell.tag .. "_" .. idx
       		
       		local descr_label = cell:find_child("descr_label", true, false)
       		if descr_label then
       			descr_label.caption = controller:get_event_progress()[idx+1].caption
       			descr_label.text_adjust_font_size = true
       		end
       		
       		if controller:get_event_progress()[idx+1].state == sqc.framework.event_progress_item_state.event_quest_completed then
       			local progress_label = cell:find_child("progress_label", true, false)
				if descr_label then
					progress_label.visible = false
				end
				
				local action_button = cell:find_child("action_button", true, false)
				if action_button then
					action_button.visible = false
				end
				
				local check = cell:find_child("check", true, false)
				if check then
					check.visible = true
				end
				
       		else
       		
				local progress_label = cell:find_child("progress_label", true, false)
				if descr_label then
					progress_label.caption = string.format(framework:strings():load("*CommonNofM", "%i of %i"), controller:get_event_progress()[idx+1].progress, controller:get_event_progress()[idx+1].count)
				end
				
				local action_button = cell:find_child("action_button", true, false)
				if action_button then
					action_button.tag = action_button.tag .. "_" .. idx;
					action_button.action =  function()
												execute_goal(idx)
											end
					
											
					local label = action_button:find_child("label", true, false)
					if label then
						if controller:get_event_progress()[idx+1].action == "make_gambling_exchange" then					
							label.caption = framework:strings():load("*CityGamblerExchange", "Exchange")
						else
							label.caption = framework:strings():load("*CityEventDialogGotoButtonTitle", "Visit")
						end
					end
					
					if controller:get_event_progress()[idx+1].state == sqc.framework.event_progress_item_state.event_quest_deny then
						action_button.active = false
					end

					local quest = framework.quests_processor:get_quest_by(controller:get_event_progress()[idx+1].quest_id)
					
					if quest.deny_text ~= "" then
						action_button.unactive_action = function()
															show_deny_msg(quest.deny_text)
														end
					end
					
				end
			end
       	end
       	       	
       	return w
	end,
	
	get_cell_count = function ( )
		return #controller:get_event_progress()
    end
}

function adjust_title_dialg()

	local dialog_title_caption = dialog:find_child('dialog_title_caption', true, false)
	if dialog_title_caption then
		local caption_rect = dialog_title_caption.rect 
		
		if g_device.is_tablet then
			if caption_rect[3] > 415 then
				caption_rect[3] = 415
				dialog_title_caption.rect = caption_rect
				dialog_title_caption.text_adjust_font_size = true
			end
		elseif g_device.is_phone then
			if caption_rect[3] > 260 then
				caption_rect[3] = 260
				dialog_title_caption.rect = caption_rect
				dialog_title_caption.text_adjust_font_size = true
			end
		end
	end
	
	
	local dialog_title_bevel = dialog:find_child('dialog_title_bevel', true, false)
	if dialog_title_bevel then
		
		local title_bevel_rect = dialog_title_bevel.rect 
		
		if g_device.is_tablet then
			if title_bevel_rect[3] > 435 then
				title_bevel_rect[3] = 435
				dialog_title_bevel.rect = title_bevel_rect
			end
		elseif g_device.is_phone then
			if title_bevel_rect[3] > 260 then
				title_bevel_rect[3] = 260
				dialog_title_bevel.rect = title_bevel_rect
			end
		end
	end
	
end

function on_post_init_dialog()
	local close_button = dialog:find_child('dialog.close_button', true, false)
	if close_button then
		close_button.action = on_close_button_action
	end	

	adjust_title_dialg()
end

function on_close_button_action()
	if controller.completed == true then
		--controller:show_goto_barn()
		show_barn_update_animation()
		framework:windows():close_all_modals()
	else
		dialog:end_dialog()
	end
end


function show_barn_update_animation()
        local reward_image = dialog:find_child('reward_image', true, false)
		if reward_image then
	        local rect=reward_image.screen_rect
             controller:show_goto_barn(rect)
		end
end

function on_proceed_button_action()

	if controller.completed == false then
		local i = 1
		
		while controller:get_event_progress()[i].state == 0 and i <= #controller:get_event_progress() do
		  i = i+1
		end
			
		if i <= #controller:get_event_progress() then
			execute_goal(i-1)
		end	
	else
        --framework:dialogs():show_storage_dialog(controller.reward_type_id)
		--controller:show_goto_barn()
		show_barn_update_animation()
		framework:windows():close_all_modals()
	end		
end

function show_deny_msg(msg)
	local params={}
	-- 0: sqc_message_box_type_default
	-- 1: sqc_message_box_type_local_error
	-- 2: sqc_message_box_type_server_error
	params.type = 0
	params.dlg_text = msg
	params.button_text = framework:strings():load("*CityOKButton","OK" )
	params.on_close_callback = close_callback
	framework:dialogs():show_message_box( params );
end