local earn_money_quests_map = {}
local earn_money_quests = {}
local gui_position = 2
local prepared_images = { ['exp'] = "achievements_list_star_on", 
                          ['gold'] = "icon_megabucks",
                          ['coins'] = "tax_big_coins_icon",
                          ['auto'] = "zp_icon",
                          ['population'] = "icon_population"
                        }

function debug_earn(s)
    print("DEBUG: " .. s)
end

function on_init_dialog()
    dialog:load_content("city_earn_money_dialog")
    update_earn_money_quests();

    local quests_table = dialog:find_child("quests_table", true, false)
    if quests_table then
        quests_table:update_content()
    end

    quests_table:scroll_to_top()
end

function update_earn_money_quests()
    local earn_money = {}

    local active_quests = framework.quests_processor:get_visible_quests()
    for i = 1, #active_quests do
        local quest_update = framework.quests_processor:get_quest_field_updates(active_quests[i])
        if quest_update then
            if quest_update.gui_position == gui_position then
                table.insert(earn_money, framework.quests_processor:get_quest_by(active_quests[i]))
            end
        end
    end

    local complited_quests = framework.quests_processor:get_complited_quests()
    for i = 1, #complited_quests do
        local quest_update = framework.quests_processor:get_quest_field_updates(complited_quests[i])
        if quest_update then 
            if quest_update.gui_position == gui_position then
                table.insert(earn_money, framework.quests_processor:get_quest_by(complited_quests[i]))
            end
        end
    end

    for i = 1, #earn_money do
        local quest_group = earn_money[i].quest_group
        if quest_group == "" then
            quest_group = earn_money[i].id
        end

        local order = earn_money[i].complete_order
        if not(earn_money_quests_map[quest_group]) then
            earn_money_quests_map[quest_group] = {}
            table.insert(earn_money_quests_map[quest_group], earn_money[i])
        elseif order then
            local is_inserted = false
            for j = 1, #earn_money_quests_map[quest_group] do
                local cur_order = earn_money_quests_map[quest_group][j].complete_order
                if not(is_inserted) and order < cur_order then
                    table.insert(earn_money_quests_map[quest_group], j, earn_money[i])
                    is_inserted = true
                end
            end

            if not(is_inserted) then
                table.insert(earn_money_quests_map[quest_group], earn_money[i])
            end
        else
            print("Error: quest with id == " .. earn_money[i].id .. " has quest_group == " .. earn_money[i].quest_group .. " and hasn't complete_order")
        end
    end

    -- print("[earn_money_quests_map]")
    for key, val in pairs(earn_money_quests_map) do
        -- print("quest_group or id == " .. key)
        local quest
        for i = 1, #val do
            -- print("id == " .. val[i].id .. ", complete_order == " .. val[i].complete_order)
            if (val[i]:is_active() or i == #val) and not(quest) then
                quest = val[i]
            end
        end

        if quest then
            local state = get_quest_state(quest:is_active(), quest.total_progress == quest.total_count)
            local is_inserted = false
            for i = 1, #earn_money_quests do
                local cur_state = get_quest_state(earn_money_quests[i]:is_active(), earn_money_quests[i].total_progress == earn_money_quests[i].total_count)
                if not(is_inserted) and (state == cur_state and quest.sort_order < earn_money_quests[i].sort_order or state < cur_state) then
                    table.insert(earn_money_quests, i, quest)
                    is_inserted = true
                end
            end

            if not(is_inserted) then
                table.insert(earn_money_quests, quest)
            end
        end
    end

    -- print("[earn_money_quests]")
    -- for i = 1, #earn_money_quests do
    --     local quest = earn_money_quests[i]
    --     print("quest with id = " .. quest.id .. ", quest_group == " .. quest.quest_group, ", complete_order == " .. quest.complete_order, ", state == " .. get_quest_state(quest:is_active(), quest.total_progress == quest.total_count) )
    -- end
end

function get_quest_state(is_active, is_completed)
    if not(is_active) then
        return 3
    elseif not(is_completed) then
        return 2
    else
        return 1
    end
end

function get_texture_name(texture)
    if g_device.is_tablet then
        texture = texture .. "_ipad"
    end
    if g_device.is_hdpi then
        texture = texture .. "@2x"
    end
    return texture
end

function get_earn_count(quest)
    local count = 0
    local event = framework.quests_processor:get_event_by_name(quest.event)
    if event then
        for i = 1, #event:get_quests_list() do
            if not(event:get_quests_list()[i].id == event.quest_event_base.id) then
                count = count + get_earn_quest_count(event:get_quests_list()[i])
            end
        end
    else
        count = get_earn_quest_count(quest)
    end
    return count
end

function get_earn_quest_count(quest)
    local count = 0
    local rolls = quest:get_rewards():get_drop_action_rolls("complete")
    for i = 1, #rolls do
        items = rolls[i]:get_items()
        for j = 1, #items do
            if items[j].type_name == "gold" then
                count = count + items[j].min_quantity
            end
        end
    end
    return count
end

function fill_rewards(cell, rewards_children_names, rewards_objects, rewards_quantities)
    if #rewards_objects > 0 then
        local icon_1 = cell:find_child(rewards_children_names.icon1, true, false)
        if icon_1 then
            local texture_name  = get_prepared_texture(rewards_objects[1].type_name)
            if not(texture_name) then texture_name = rewards_objects[1].icon_name end
            if not(texture_name) or string.len(texture_name) == 0 then
                texture_name = rewards_objects[1].image_path
            end
            icon_1.texture_name = texture_name
        end

        local caption_1 = cell:find_child(rewards_children_names.caption1, true, false)
        if caption_1 then
            caption_1.caption = rewards_quantities[1]
        end

        if #rewards_objects > 1 then
            local icon_2 = cell:find_child(rewards_children_names.icon2, true, false)
            if icon_2 then
                local texture_name  = get_prepared_texture(rewards_objects[2].type_name)
                if not(texture_name) then texture_name = rewards_objects[2].icon_name end
                if not(texture_name) or string.len(texture_name) == 0 then
                    texture_name = rewards_objects[2].image_path
                end
                icon_2.texture_name = texture_name
            end

            local caption_2 = cell:find_child(rewards_children_names.caption2, true, false)
            if caption_2 then
                caption_2.caption = rewards_quantities[2]
            end
        end

        if #rewards_objects > 2 then
            local icon_3 = cell:find_child(rewards_children_names.icon3, true, false)
            if icon_3 then
                local texture_name  = get_prepared_texture(rewards_objects[3].type_name)
                if not(texture_name) then texture_name = rewards_objects[3].icon_name end
                if not(texture_name) or string.len(texture_name) == 0 then
                    texture_name = rewards_objects[3].image_path
                end
                icon_3.texture_name = texture_name
            end

            local caption_3 = cell:find_child(rewards_children_names.caption3, true, false)
            if caption_3 then
                caption_3.caption = rewards_quantities[3]
            end
        end
    end
end

function fill_stars(cell, complete_order, complete_order_count, quest_completed)
    if complete_order_count > 1 then
        local ern_star_des_2 = cell:find_child("ern_star_des_2", true, false)
        if ern_star_des_2 then
            ern_star_des_2:set_visible(true)
        end
    end

    if complete_order_count > 2 then
        local ern_star_des_3 = cell:find_child("ern_star_des_3", true, false)
        if ern_star_des_3 then
            ern_star_des_3:set_visible(true)
        end
    end

    if complete_order > 1 or complete_order == 1 and quest_completed then
        local ern_star_gold_1 = cell:find_child("ern_star_gold_1", true, false)
        if ern_star_gold_1 then
            ern_star_gold_1:set_visible(true)
        end
    end

    if complete_order > 2 or complete_order == 2 and quest_completed then
        local ern_star_gold_2 = cell:find_child("ern_star_gold_2", true, false)
        if ern_star_gold_2 then
            ern_star_gold_2:set_visible(true)
        end
    end

    if complete_order == 3 and quest_completed then
        local ern_star_gold_3 = cell:find_child("ern_star_gold_3", true, false)
        if ern_star_gold_3 then
            ern_star_gold_3:set_visible(true)
        end
    end
end

function on_cell_earn_action_continue(quest)
    print("[city_earn_money_dialog] earn action continue")

    if quest then
        local event = framework.quests_processor:get_event_by_name(quest.event)
        if event then
            framework:dialogs():show_choose_event_stage(quest.id)
        else
            framework:dialogs():show_choose_quest(quest.id)
        end
    end

    dialog:end_dialog()
end

function on_cell_earn_action_take_reward(quest)
    print("[city_earn_money_dialog] earn action take reward")

    if quest then
        framework.quests_processor:end_non_auto_complete_quest(quest.id)
        framework.quests_processor:update()
    end

    dialog:end_dialog()
end

-- quests table delegate

quests_table_delegate = {}

function quests_table_delegate:create_cell(idx, rect)
    if #earn_money_quests == 0 then
        print("[city_earn_money_dialog] earn money quests size is 0")
        return nil
    end

    local index = idx + 1
    local quest = earn_money_quests[index]

    if not(quest) then
        print("[city_earn_money_dialog] quest object is null")
        return nil
    end

    local cell = self.table:create_child(rect)
    framework:json_ui_manager():build_from_resource("city_earn_money_cell", cell)
    
    local cell_image = cell:find_child("cell_image", true, false)
    if cell_image then
        cell_image.texture_name = get_texture_name(quest.image)
    end

    local caption_text = quest.name;
    local quest_update = framework.quests_processor:get_quest_field_updates(quest.id)
    if quest_update and quest_update.skip_goal_dialog then
    	local n = quest_update.goal_for_action;
    	local goal =  quest:get_goal(n)
    	if goal then
    	   caption_text = goal.description
    	end
    end

    local cell_caption = cell:find_child("cell_caption", true, false)
    if cell_caption then
        cell_caption.caption = caption_text
    end

    local rolls = quest:get_rewards():get_drop_action_rolls("complete")
    local items = rolls[1]:get_items()

    local rewards_objects = {}
    local rewards_quantities = {}
    for i = 1, #items do
       rewards_objects[i] = framework:object_factory():get_object_info_by_type_name(items[i].type_name)
       rewards_quantities[i] = items[i].min_quantity
    end

    local quest_group = quest.quest_group
    local complete_order = 1
    local complete_order_count = 1

    if quest.complete_order > complete_order then
        complete_order = quest.complete_order
    end

    if quest_group ~= "" then
        for key, val in pairs(earn_money_quests_map) do
            if key == quest_group then
                complete_order_count = #val
            end
        end
    end

    local quest_progress = quest.total_progress
    local quest_total_count = quest.total_count
    local quest_completed = (quest_progress == quest_total_count) or not(quest:is_active())

    fill_stars(cell, complete_order, complete_order_count, quest_completed)
    
    if quest:is_active() then
        local cell_reward_active = cell:find_child("cell_reward_active", true, false)
        if cell_reward_active then
            cell_reward_active:set_visible(true)
        end

        local rewards_children_names = {icon1 = 'reward_icon_active_1', caption1 = 'reward_caption_active_1',
                                        icon2 = 'reward_icon_active_2', caption2 = 'reward_caption_active_2',
                                        icon3 = 'reward_icon_active_3', caption3 = 'reward_caption_active_3'}

        fill_rewards(cell, rewards_children_names, rewards_objects, rewards_quantities)

        local cell_action_continue = function() on_cell_earn_action_continue(quest) end
        local cell_action_take_reward = function() on_cell_earn_action_take_reward(quest) end

        local cell_bg = cell:find_child("cell_bg", true, false)

        local cell_active_quest_progress = cell:find_child("cell_active_quest_progress", true, false)
        if cell_active_quest_progress then
            cell_active_quest_progress:set_visible(true)

            if quest_completed then
                local ern_complited_caption = cell:find_child("ern_complited_caption", true, false)
                if ern_complited_caption then
                    ern_complited_caption:set_visible(true)
                end
                
                local ern_button_take_reward = cell:find_child("ern_button_take_reward", true, false)
                if ern_button_take_reward then
                    ern_button_take_reward:set_visible(true)
                    ern_button_take_reward.action = cell_action_take_reward
                end

                if cell_bg then
                    cell_bg.action = cell_action_take_reward
                end
            else
                local ern_progress_bg = cell:find_child("ern_progress_bg", true, false)
                if ern_progress_bg then
                    ern_progress_bg:set_visible(true)

                    local ern_progress = cell:find_child("ern_progress", true, false)
                    if ern_progress and quest_progress > 0 then
                        ern_progress.range = { 0, 100 * quest_total_count / quest_progress }
                        ern_progress.pos = 100
                    end

                    local ern_progress_caption = cell:find_child("ern_progress_caption", true, false)
                    if ern_progress_caption then
                        ern_progress_caption.caption = string.format(ern_progress_caption.caption, quest_progress, quest_total_count)
                    end
                end

                local ern_button_continue
                if quest_progress > 0 then
                    ern_button_continue = cell:find_child("ern_button_continue", true, false)
                else
                    ern_button_continue = cell:find_child("ern_button_complete", true, false)
                end

                if ern_button_continue then
                    ern_button_continue:set_visible(true)
                    ern_button_continue.action = cell_action_continue
                end

                if cell_bg then
                    cell_bg.action = cell_action_continue
                end
            end
        end
    else
        local cell_completed_quest_info = cell:find_child("cell_completed_quest_info", true, false)
        if cell_completed_quest_info then
            cell_completed_quest_info:set_visible(true)
        end

        local rewards_children_names = {icon1 = 'reward_icon_completed_1', caption1 = 'reward_caption_completed_1',
                                        icon2 = 'reward_icon_completed_2', caption2 = 'reward_caption_completed_2',
                                        icon3 = 'reward_icon_completed_3', caption3 = 'reward_caption_completed_3'}

        fill_rewards(cell, rewards_children_names, rewards_objects, rewards_quantities)
    end

    return cell
end


function quests_table_delegate:get_cell_count()
    return #earn_money_quests
end

function get_prepared_texture(type)
    for key, val in pairs(prepared_images) do
        if key == type then
            return get_texture_name(val)
        end
    end
    return nil
end