
function load_configuration(file_name, sandbox) 
    local data = framework:open_file_from_bundle(file_name)
    if not data then 
        print("[city_requests_dialog] can't find configuration: " .. file_name )
        return false
    end
    if not scripting_server:load_stream( file_name, data , sandbox ) then
        print("[city_requests_dialog] failed loading configuration: " .. file_name )
        return false
    end
    print("[city_requests_dialog] configuration is loaded")
    return true
end

local configuration_loaded = false

local MAX_CATEGORY_SCROLL_CELLS = 3
local MAX_CATEGORY_SCROLL_CELLS_TABLE = 4

local auto_accept_params = { active = false, response = false }

local category_infos = {}

local is_dialog_destroyed = false
local content
local content_scroll = {index = nil}
local room_specific_requests = {}

local category_animation_params = nil
local request_animate_params = nil

local barn_animator=nil
local barn_animator_params=nil

local push_time_hash={}

function init_barn_animator()
    if not barn_animator then
        local animator_name = "city_barn_animation_impl"
        local barn_animator_class = cclasses[animator_name]
        if barn_animator_class then        
            barn_animator=barn_animator_class.new()
        end
    end

    if not barn_animator_params then
        local animator_params_name="barn_animator_params"
        local barn_animator_params_class=cclasses[animator_params_name]
        if barn_animator_params_class then 
            barn_animator_params=barn_animator_params_class.new()
        end
    end
end

local send_gift_cache = {}
function reset_send_gift_cache(friend_id)
    send_gift_cache[friend_id] = nil
end

function get_send_gift(friend_id)
    local gift = send_gift_cache[friend_id]
    if gift then
        return gift
    end
    gift = requests_controller:get_send_gift_info(friend_id)
    if gift then
        send_gift_cache[friend_id] = gift
    end
    return gift
end

function on_init_dialog()
    configuration_loaded = load_configuration("city_requests_config.lua")
    dialog:load_content("city_requests_dialog")
    configurate_dialog()

    update_interface_for_auto_accept_feature()

    requests_controller:set_refresh_callback(refresh_active_requests)
    refresh_active_requests()
end

function update_dialog()
	refresh_active_requests()
end

function configurate_dialog()
    if configuration_loaded then
        local requests_limit_label = dialog:find_child("requests_day_limit_label", true, false)
        if requests_limit_label then
            requests_limit_label.visible = is_requests_day_limit_label()
        end 
    end
end

function on_post_init_dialog()
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        requests_table.scroll_pos = {0, requests_table.cell_height}
    end

    init_barn_animator()
    update_interface_for_room_specific()
end


function end_dialog()
    requests_controller:set_refresh_callback(nil)
    lock_interface(false)
    cancel_accept_request_animation()
    is_dialog_destroyed = true
end

function cancel_accept_request_animation()
    if request_animate_params then
        if request_animate_params.current_transform then
            framework:windows():animation():cancel_animation(request_animate_params.current_transform)
        end
        if request_animate_params.next_transform then
            framework:windows():animation():cancel_animation(request_animate_params.next_transform)
        end
    end
    request_animate_params = nil
end

function notify_callback(provider_id, event_id)
    if provider_id == "country_provider_id" then
        if event_id == "country_room_changed" then 
            refresh_active_requests()
            update_interface_for_room_specific()
            local requests_table = dialog:find_child("requests_table", true, false)
            if requests_table then
                requests_table.scroll_pos = {0, requests_table.cell_height}
            end
        end
    end
end

function get_dialog_init_arg()
    local params = {}
    if g_device.is_tablet then
        params.title_args_bevel_width = 255
        params.title_args_bevel_offset = 5
    else
        params.title_args_bevel_width = 170
        params.title_args_bevel_offset = 7
    end
    return params
end

function update_interface_for_room_specific()
    if room_specific_feature_available then
        local room_id = framework:game().room_id
        local rooms = rooms_api.get_rooms_info()
        for i = 1, #rooms do
            if room_id == rooms[i].id then
                local dialog_caption = dialog:find_child("dialog_title_caption", true, false)
                if dialog_caption then
                    dialog_caption.caption = rooms[i].name
                end
                break
            end
        end

        local is_room_specific_button = false
        for i = 1, #rooms do
            if rooms[i].opened and not(rooms[i].id == framework:game().room_id) then
                is_room_specific_button = true
                break
            end
        end
        
        if is_room_specific_button then
            local requests_room_specific_button = dialog:find_child("requests_room_specific", true, false)
            if requests_room_specific_button then
                requests_room_specific_button.visible = true
                local specific_icon = requests_room_specific_button:find_child("requests_room_specific_icon", true, false)
                if specific_icon then
                    specific_icon.visible = #room_specific_requests == 0
                end
                local specific_icon_counter = requests_room_specific_button:find_child("requests_room_specific_icon_counter", true, false)
                if specific_icon_counter then
                   specific_icon_counter.visible = #room_specific_requests > 0 
                end
            end
        end
    end
end

function update_interface_for_auto_accept_feature()
    if not(auto_accept_feature_available) then
        local accept_all_button = dialog:find_child("requests_accept_all_button", true, false)
        if accept_all_button then
            accept_all_button.visible = false
            local bevel = dialog:find_child("requests_bevel", true, false)
            if bevel then
                local rect = bevel.rect
                rect[4] = rect[4] + accept_all_button.rect[4]
                rect[2] = rect[2] - accept_all_button.rect[4]
                bevel.rect = rect
            end
            local requests_table = dialog:find_child("requests_table", true, false)
            if requests_table then
                local rect = requests_table.rect
                rect[4] = rect[4] + accept_all_button.rect[4]
                rect[2] = rect[2] - accept_all_button.rect[4]
                requests_table.rect = rect
            end
        end
    end
end

function update_auto_accept_params(request)
    if auto_accept_feature_available then
        local info = framework.requests_processor:get_request_info(request.name)
        if not(auto_accept_params.active) then
            auto_accept_params.active = check_can_accept_today(info) and 
               --not(requests_controller:allow_auto_accept(request) == sqc.requests.sqc_request_availability.sqc_request_disabled) 
               not(check_auto_accept_allowed(request,info) == sqc.requests.sqc_request_availability.sqc_request_disabled) 
        end
        if not(auto_accept_params.response) then
            auto_accept_params.response = is_ask_request(request, info)
        end
    end
end

function check_auto_accept_allowed(request,request_info)
	if request_info then
		if(request_info.accept_action == sqc.requests.sqc_request_accept_action.request_accept_action_visit_farm) then
        	return sqc.requests.sqc_request_availability.sqc_request_disabled
    	elseif(	request_info.name == "stock_exchange_request") then
        	return sqc.requests.sqc_request_availability.sqc_request_disabled
    	elseif(	request_info.name == "build_bus_station") then
        	return sqc.requests.sqc_request_availability.sqc_request_disabled
    	elseif(	request_info.name == "invite_suggested_neighbors" or
          		request_info.name == "invite_neighbors" or
            	request_info.name == "remove_neighbors"
		) then
        	return sqc.requests.sqc_request_availability.sqc_request_disabled
    	elseif(	request_info.name == "build_train_station") then
        	return sqc.requests.sqc_request_availability.sqc_request_disabled
    	end
    end
    if request then
		return requests_controller:allow_accept(request)
	else
		return sqc.requests.sqc_request_availability.sqc_request_hidden
	end
end

function update_category_infos()
    
    query_infos()
    sort_info()
    
    answers_available = requests_controller:get_answers_available()
end

function init_category_info(request_category)
    return
    {
        category = request_category,
        requests = {},
        open = true,
        height = 0,
        height_full = 0,
        scroll = {scrollable = false, left = 0, right = 0}
    }
end

function query_infos()

    local states = {}
    for i = 1, #category_infos do
        local info = category_infos[i]
        local state = {}
        state.open = info.open
        state.scroll = info.scroll
        state.height = info.height_full
        if info.container then
            state.scroll.pos = info.container.scroll_pos[2]
        end
        states[category_infos[i].category.tag] = state
    end

    category_infos = {}
    room_specific_requests = {}

    auto_accept_params = { active = false, response = false }

    local infos = {}
    for i = 1, requests_controller:get_request_count() do
        local request = requests_controller:get_request(i - 1)
        
        local state = requests_controller:allow_accept(request)
        if state == sqc.requests.sqc_request_availability.sqc_request_available or 
           state == sqc.requests.sqc_request_availability.sqc_request_disabled then
            local category = framework.requests_processor:get_category(request)
            if not infos[category.tag] then
                infos[category.tag] = init_category_info(category)
                local state = states[category.tag]
                if state then
                    infos[category.tag].open = state.open
                    infos[category.tag].scroll = state.scroll
                    infos[category.tag].old_full_height = state.height
                end
                category_infos[#category_infos + 1] = infos[category.tag]
            end
            table.insert(infos[category.tag].requests, request)
        elseif state == sqc.requests.sqc_request_availability.sqc_request_room_specific then
            table.insert(room_specific_requests, request)
        end

        update_auto_accept_params(request)
    end
end

function requests_sorting_functor(a,b)
    if(a.push_time ~= b.push_time) then
        return a.push_time > b.push_time 
    else
        return a.order_number < b.order_number 
    end
end


function update_requests_order_numbers()
    for i = 1, #category_infos do
      for j = 1, #category_infos[i].requests do
         local request = category_infos[i].requests[j]

         if request.order_number==0 then

            local max_order_number=push_time_hash[request.push_time]

            if max_order_number then 
                push_time_hash[request.push_time]=max_order_number+1
            else
                push_time_hash[request.push_time]=1
            end
            
            request.order_number=push_time_hash[request.push_time]
         end
      end 
    end

end



function sort_info()
    table.sort(category_infos, function(a, b) return  a.category.order < b.category.order end)

    for i = 1, #category_infos do
        table.sort(category_infos[i].requests,  requests_sorting_functor )
    end

    update_requests_order_numbers()

end

function refresh_active_requests()
	if is_dialog_destroyed then
		do return end
	end
	
    if dialog then
        update_category_infos()
        update_content()
    end
end

function close_window()
    can_close_window=true
    if barn_animator and barn_animator.animation_started then
        can_close_window=false
    end

    if can_close_window then
        dialog:end_dialog()
    end
end


function update_content()
      
    create_content()

    local requests_limit_label = dialog:find_child("requests_day_limit_label", true, false)
    if requests_limit_label and requests_limit_label.visible then
        if not (answers_available == 0) then
            local caption = framework:strings():load("*CityRequestsAllowedAnswers", "You can answer to %d letters")
            requests_limit_label.caption = string.format(caption, answers_available)
        else
            local caption = framework:strings():load("*RequestEmptyAvailabel", "Empty availabel")
            requests_limit_label.caption = caption
        end
    end
    update_accept_all_requests_button()

    if #category_infos == 0 then
        if #room_specific_requests > 0 then
            show_reqeusts_empty_plate()
        else
            close_window()
        end  
    else 
        show_requests_table()
    end 
end

function is_ask_material(info)
    return info.material and (info.type == "common" or info.type == "personal")
end

function is_ask_item(info)
    return info.type == "personal" and info.subtype == "item_query"
end

function is_ask_response(request)
    return request.waiting_state == sqc.requests.sqc_request_waiting_status.waiting_callback
end

function is_ask_response_thanks(info)
    return info.callback_request == "say_thanks"
end

function is_ask_fuel(info)
    return info.subtype == "request_fuel"
end

function is_ask_request(request, info)
    return  is_ask_material(info) or is_ask_item(info) or
            is_ask_response(request) or is_ask_fuel(info)
end

function is_accept_contract_help(info)
    return info.type == "common" and (info.subtype == "help_contract" or info.subtype == "request_recover_contract")
end

function is_accept_social_contract_help(info)
    return info.type == "visit" and info.subtype == "help_contract"
end

function is_accept_staff(info)
    return info.type == "staff"
end

function is_accept_gift(info)
    return info.type == "personal" and (info.name == "send_gift" or info.name == "send_gift_new")
end

function is_accept_material(info)
    return info.type == "material"
end

function is_accept_animal(info)
    return info.type == "animal"
end

function is_accept_transport(info)
    return info.type == "transport"
end

function is_accept_transport_back(info)
    return info.type == "transport_back"
end

function get_right_item_rect()
    if g_device.is_tablet then
        return {4, 30, 114, 58}
    end
    return {2, 18, 96, 52}
end

function get_category_caption_size()
    if g_device.is_tablet then
        return 50
    end
    return 30
end

function get_post_description(post)
    local caption = ""
    local message = post

    local index = string.find(post, "\n\n")
    if index then
        caption = string.sub(post, 1, index)
        message = string.sub(post, index + 2)
    end
    string.gsub(message, "\n", " ")
    return caption .. message
end

function get_image_resource(image)
    if g_device.is_tablet then
        image = image .. "_ipad"
    end
    return image
end

function show_requests_table()
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        requests_table.visible = true
    end
    local empty_plate = dialog:find_child("requests_empty_plate", true, false)
    if empty_plate then
        empty_plate.visible = false
    end
end

function show_reqeusts_empty_plate()
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        requests_table.visible = false
    end
    local empty_plate = dialog:find_child("requests_empty_plate", true, false)
    if empty_plate then
        empty_plate.visible = true
    end
end

function show_reward(cell)
    local cell_right_item_bg = cell:find_child("cell_right_item_bg", true, false)
    if cell_right_item_bg then
        cell_right_item_bg.visible = true;
    end
end

function build_reward_item(value, image, parent)

    local reward = framework:json_ui_manager():create_from_resource("city_requests_reward_item", parent)
    if not(reward) then
        print("[city_requests_dialog] can't create reward item")
        do return end
    end

    local reward_value = reward:find_child("reward_value", true, false)
    if reward_value then
        reward_value.caption = "x" .. tostring(value)
    end

    local reward_image = reward:find_child("reward_image", true, false)
    if reward_image then
        reward_image.image_name = get_image_resource(image)
    end
    return reward
end

function layout_reward_items(rewards)
    local spaces = 4
    local rect = get_right_item_rect()
    local reward_rect = rewards[1].rect
    local width = (rect[3] - (#rewards * (reward_rect[3] + spaces))) / 2
    local height = (rect[4] - reward_rect[4]) / 2
    local position = {rect[1] + width + spaces, rect[2] + height}
    for i = 1, #rewards do
        local rect = {position[1], position[2], reward_rect[3], reward_rect[4]}
        rewards[i].rect = rect
        position[1] = position[1] + reward_rect[3] + spaces
    end
end

function smudge_reward_item(reward)
    local reward_value = reward:find_child("reward_value", true, false)
    if not(reward_value) then
        do return end
    end

    local reward_image = reward:find_child("reward_image", true, false)
    if not(reward_image) then
        do return end
    end

    local rect = get_right_item_rect()
    reward.rect = rect

    local reward_image_rect = reward_image.rect
    local width = (rect[3] - reward_image_rect[3]) / 2
    local height = (rect[4] - reward_image_rect[4]) / 2
    reward_image_rect[1] = width
    reward_image_rect[2] = height
    reward_image.rect = reward_image_rect

    local reward_value_rect = reward_value.rect
    reward_value_rect[1] = rect[3] - reward_value_rect[3]
    reward_value.rect = reward_value_rect

end

function update_reward_items(info, data)

    local cell_right_item_bg = data.cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    local reward_images = {}
    local reward_values = {}
    for i = 1, #info:get_reward() do

        local exp_item = info:get_reward()[i]:get_item("exp")
        local coins_item = info:get_reward()[i]:get_item("coins")
        local respect_item = info:get_reward()[i]:get_item("respect")
        local gold = info:get_reward()[i]:get_item("gold")

        if exp_item then
            table.insert(reward_images, "icon_level")
            table.insert(reward_values, exp_item.min_quantity)
        end
        if coins_item then
            table.insert(reward_images, "icon_coins")
            table.insert(reward_values, coins_item.min_quantity)
        end 
        if respect_item then
            table.insert(reward_images, "respect_heart")
            table.insert(reward_values, respect_item.min_quantity)
        end
        if gold then
            table.insert(reward_images, "reg_icon_bucks")
            table.insert(reward_values, gold.min_quantity)
        end
    end

    local rewards = {}
    for i = 1, #reward_images do
        table.insert(rewards, build_reward_item(reward_values[i], reward_images[i], cell_right_item_bg))
    end

    if #rewards > 1 then
        layout_reward_items(rewards)
    elseif #rewards == 1 then
        smudge_reward_item(rewards[1])
    end

    if not(#reward_images == 0) then
        show_reward(data.cell)
        data.need_show_reward = true
    end
end

function update_reward_transport_items(request, data)
    local cell_right_item_bg = data.cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    local count = request.calculated_reward
    if count > 0 then
        local reward = build_reward_item(count, "icon_coins", cell_right_item_bg)
        smudge_reward_item(reward)
        show_reward(data.cell)
        data.need_show_reward = true
    end

end

function update_request_item(image, data)

    local cell_right_item_bg = data.cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    local texture_proxy = framework:json_ui_manager():create_widget(cell_right_item_bg, "sqc_proxy_texture_widget", get_right_item_rect())
    texture_proxy:set_ready_progress_as_indicator()
    texture_proxy.texture_name = image

    data.need_show_reward = true
    show_reward(data.cell)
end

function update_request_item_count(count, cell)
    
    local cell_right_item_bg = cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    local material_count = cell_right_item_bg:find_child("cell_material_count", true, false)
    if material_count and count>1 then
        material_count.visible = true
        material_count.caption = "x" .. tostring(count)
    end
end

function update_request_item_properties(info, cell)
    local cell_right_item_bg = cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end
    
    local reward_images = {}
    local reward_values = {}

    for i = 1, #info:get_reward() do 
        local item_xp = info:get_reward()[i]:get_item("exp")
        local item_coins = info:get_reward()[i]:get_item("coins")
        if item_xp then
            table.insert(reward_images, "xp_icon")
            table.insert(reward_values, item_xp.min_quantity)
        end
        if item_coins then
            table.insert(reward_images, "coins_icon")
            table.insert(reward_values, item_coins.min_quantity)
        end
    end

    local propertis = 
    {
        cell_right_item_bg:find_child("cell_property_1", true, false),
        cell_right_item_bg:find_child("cell_property_2", true, false)
    }

    for i = 1, #reward_images do
        if propertis[i] then
            propertis[i].image_name = get_image_resource(reward_images[i])
            propertis[i].visible = true
            local property_value = propertis[i]:find_child("property_value", true, false)
            if property_value then
                property_value.caption = "+" .. tostring(reward_values[i])
            end
            propertis[i]:raise()
        end
    end
end

function update_request_transport_item_properties(reqeust, cell)
    local cell_right_item_bg = cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    local count = reqeust.calculated_reward
    if count > 0 then
        local propertis = cell_right_item_bg:find_child("cell_property_1", true, false)
        if propertis then
            propertis.image_name = get_image_resource("coins_icon")
            propertis.visible = true
            local property_value = propertis:find_child("property_value", true, false)
            if property_value then
                property_value.caption = "+" .. tostring(count)
            end
            propertis:raise()
        end
    end
end

function update_contract_item(info, cell)
    local cell_right_item_bg = cell:find_child("cell_right_item_bg", true, false)
    if not cell_right_item_bg then
        do return end
    end

    if info.subtype == "help_contract" then
        local help_icon = cell_right_item_bg:find_child("cell_help_icon", true, false)
        if help_icon then
            help_icon.visible = true
        end
    end

    if info.subtype == "request_recover_contract" then
        local help_recover = cell_right_item_bg:find_child("cell_recover_icon", true, false)
        if help_recover then
            help_recover.visible = true
        end
    end
end

function check_can_accept_today(info)
    if not(info.without_per_day_limit) then
        return not (answers_available == 0)
    end
    return true
end

function is_action_active(data)
    if requests_controller:allow_accept(data.request) == sqc.requests.sqc_request_availability.sqc_request_disabled then 
        return false
    end
    return check_can_accept_today(data.info)
end

function is_need_callback(request)
    local profile = framework:friends():find_friend(request.friend_id, true);
    if not(profile) or not(profile.is_in_game) or profile.is_bot then
        return false
    end 
    
    local info = framework.requests_processor:get_request_info(request.name)
    return not(info.auto_call_back) and not(is_ask_response(request))
end

function update_accept_all_requests_button()
print("update_accept_all_requests_button() called")
    local accept_all_button = dialog:find_child("requests_accept_all_button", true, false)
    if accept_all_button and accept_all_button.visible then
    	if #category_infos == 0 then
    		accept_all_button.active = false
    	else
        	accept_all_button.active = auto_accept_params.active
     	end
        local accept_all_caption = accept_all_button:find_child("requests_accept_all_label")
        if accept_all_caption then
            if auto_accept_params.response then
                accept_all_caption.caption = framework:strings():load("*CityAcceptAllRequestsAndSend", "Accept all requests and send gifts")
            else
                accept_all_caption.caption = framework:strings():load("*CityAcceptAllRequests", "Accept all requests")
            end
        end
    end
end

function update_cell_action(data)

    local active = is_action_active(data)
    local accept_action = function() on_cell_accept_action(data) end
    local accept_unactive_action = function() on_cell_accept_unactive_action(data) end
    local use_request_object = function() on_cell_use_request_object(data) end

    if is_ask_request(data.request, data.info) or data.is_callback then
        local button_accept = data.cell:find_child("cell_accpet", true, false)
        if button_accept then
            button_accept.visible = false
        end
        local button_send = data.cell:find_child("cell_send", true, false)
        if button_send then
            button_send.visible = true
            button_send.action = accept_action
            button_send.unactive_action = accept_unactive_action
            button_send.active = active
            if not(active) then
                local image = button_send:find_child("cell_send_image", true, false)
                if image then
                    image.image_name = get_image_resource("requests_disabled_send")
                end
            end

            local button_send_label = button_send:find_child("cell_send_label", true, false)
            if button_send_label then
                button_send_label.caption = data.info.receive_button
            end
        end
    else
        local is_storable = is_storable_gift(data)
        local button_accept = data.cell:find_child("cell_accpet", true, false)
        if button_accept then
            if not is_storable then
                button_accept.action = use_request_object
            else
                button_accept.action = accept_action
                button_accept.unactive_action = accept_unactive_action
            end
            button_accept.active = active
            if not(active) then
                local image = button_accept:find_child("cell_accept_image", true, false)
                if image then
                    image.image_name = get_image_resource("requests_disabled_ok")
                end
            end

            local button_accept_lable = button_accept:find_child("cell_accept_label", true, false)
            if button_accept_lable then
                if is_storable then
                    button_accept_lable.caption = data.info.receive_button
                else
                	if is_auto_use_gift(data) then
                	    button_accept_lable.caption = framework:strings():load("*CommonUseButtonText", "Apply");
                	else
                		button_accept_lable.caption = framework:strings():load("*CityOk", "Ok")
                	end
                end
            end
        end
        
        local button_receive = data.cell:find_child("cell_receive", true, false)
        if button_receive and is_storable then
--        	print("data.request.has_additional_button = ", data.request.has_additional_button)
        	if data.request.has_additional_button == true then
        		button_receive.visible = true
				button_receive.action = use_request_object
        		button_receive.active = active
        		
				local button_receive_lable = button_receive:find_child("cell_receive_label", true, false)
				if button_receive_lable then
					local s = data.info.use_button
					if is_buildable_gift(data) == true then
                                                if not data.request.name=="send_gift_new" then
						    button_receive.visible = false
						end
						s = data.info.build_button
					end
					if s == nil or s == '' or s == "Use" then
						s = framework:strings():load("*CommonUseButtonText", "Apply")
					end
					button_receive_lable.caption = s
		    	end
        	end
        end
        
    end
end

function is_buildable_gift(data)
        local item_info = framework:object_factory():get_object_info_by_type_id(data.request.item_id)
        if item_info then
		if item_info.requestable == true
		   or item_info.shop_department == "bouquets"
		   or item_info.shop_department == "act_of_expanding"
		   or item_info.shop_department == "expand"
		   or item_info.shop_department == "materials"
		   or item_info.shop_department == "craft_materials"
		   or item_info.shop_department == "fuels"
		then
			return false
		end

		return true
	end

	return false
end

function is_storable_gift(data)
    local item_info = framework:object_factory():get_object_info_by_type_id(data.request.item_id)
    if item_info then
        if (item_info.superclass_name == 'automate_base' and item_info.shop_department == 'automation')
        	or (item_info.superclass_name == 'bouquet_base' and item_info.shop_department == 'bouquets') then
            return false
        end
    end
    return true
end

function is_auto_use_gift(data)
    local item_info = framework:object_factory():get_object_info_by_type_id(data.request.item_id)
    if item_info then
        if item_info.superclass_name == 'bouquet_base' and item_info.shop_department == 'bouquets' then
            return true
        end
    end
    return false
end

function update_cell_cancel(data)
    local button_cancel = data.cell:find_child("cell_cancel", true, false)
    if button_cancel then
        button_cancel.visible = data.info.can_discard
        button_cancel.action = function() on_cell_cancel_action(data) end
        local cancel_button_text = data.info.receive_cancel_button
        if not(cancel_button_text == "") then
            local button_cancel_label = button_cancel:find_child("cell_cancel_label", true, false)
            if button_cancel_label then
                button_cancel_label.caption = cancel_button_text
            end
        end
    end
end

function update_cell_callback_cancel(data)
    local button_cancel = data.cell:find_child("cell_cancel", true, false)
    if button_cancel then
        button_cancel.visible = data.info.can_call_back_discard
        button_cancel.action = function() on_cell_cancel_action(data) end
        local cancel_button_text = data.info.receive_cancel_button
        if not(cancel_button_text == "") then
            local button_cancel_label = button_cancel:find_child("cell_cancel_label", true, false)
            if button_cancel_label then
                button_cancel_label.caption = cancel_button_text
            end
        end
    end
end

function update_cell_callback_action(data)
    local active = is_action_active(data)
    local button_accept = data.cell:find_child("cell_accpet", true, false)
    if button_accept then
        button_accept.visible = false
    end
    local button_send = data.cell:find_child("cell_send", true, false)
    if button_send then
        button_send.visible = true
        button_send.action = function() on_cell_accept_action(data) end
        button_send.unactive_action = function() on_cell_accept_unactive_action(data) end
        button_send.active = active
        if not(active) then
            local image = button_send:find_child("cell_send_image", true, false)
            if image then
                image.image_name = get_image_resource("requests_disabled_send")
            end
        end

        local button_send_label = button_send:find_child("cell_send_label", true, false)
        if button_send_label then
            if is_ask_response_thanks(data.info) and get_send_gift(data.request.friend_id) then
                button_send_label.caption = data.info.send_button
            else
                local callback_info = framework.requests_processor:get_request_info(data.info.callback_request)
                if callback_info then
                    button_send_label.caption = callback_info.send_button
                end
            end
        end
    end
end

function update_cell_item_action(data)
    local cell_right_item_button = data.cell:find_child("cell_right_item_bg", true, false)
    if cell_right_item_button then
        cell_right_item_button.action = function() on_cell_accept_action(data) end
        cell_right_item_button.unactive_action = function() on_cell_accept_unactive_action(data) end
        cell_right_item_button.active = is_action_active(data)
    end
end

function update_cell_items(data)
    if is_ask_material(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_count(1, data.cell)
    elseif is_ask_item(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_count(1, data.cell)
    elseif is_ask_fuel(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_properties(data.info, data.cell)
    elseif is_accept_staff(data.info) then
        local position = framework.requests_processor:get_staff_position(data.request);
        local position_info = framework:object_factory():get_object_info_by_type_name(position)
        if position_info then
            update_request_item(position_info.image_path, data)
            update_request_item_properties(data.info, data.cell)
        end
    elseif is_accept_contract_help(data.info) then
        local contract_id = data.request.contract_id
        if not(contract_id == 0) then
            local contract_info = framework:object_factory():get_object_info_by_type_id(contract_id)
            if contract_info then
                update_request_item(contract_info.shop_image_path, data)
                update_contract_item(data.info, data.cell)
            end
        end
    elseif is_accept_social_contract_help(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_properties(data.info, data.cell)
    elseif is_accept_gift(data.info) then
        local item_info = framework:object_factory():get_object_info_by_type_id(data.request.item_id)
        if item_info then
            update_request_item(item_info.shop_image_path, data)
            update_request_item_count(1, data.cell)
        end
    elseif is_accept_transport(data.info) then
        update_request_item(data.request.image, data)
        update_request_transport_item_properties(data.request, data.cell)
    elseif is_accept_transport_back(data.info) then
        update_reward_transport_items(data.request, data)
    elseif is_accept_material(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_count(1, data.cell)
    elseif is_accept_animal(data.info) then
        update_request_item(data.request.image, data)
        update_request_item_count(1, data.cell)
    else
        update_reward_items(data.info, data)
    end
end

function update_cell_callback_items(data)
    local gift_info = get_send_gift(data.request.friend_id)
    if gift_info then
        update_request_item(gift_info.image_path, data)
        update_request_item_count(1, data.cell)
    else
        local callback_request = framework.requests_processor:create_callback_request(data.request)
        if callback_request then
            local callback_request_info = framework.requests_processor:get_request_info(callback_request.name)
            if callback_request_info then
                update_reward_items(callback_request_info, data)
            end
        end
    end
end

function update_cell_info(data)
    local user_name = social_api.get_normalized_user_name(data.profile, 0)
    local user_gender = sqc.framework.sqc_male
    if data.profile then
        user_gender = data.profile.gender
    end


    local cell_info
    if data.need_show_reward == true then
        cell_info = data.cell:find_child("cell_info_with_reward", true, false)
    else
        cell_info = data.cell:find_child("cell_info", true, false)
    end
    if cell_info then
        cell_info.visible = true
        if is_ask_response_thanks(data.info) and get_send_gift(data.request.friend_id) then
            cell_info.caption = get_post_description(data.request:get_formatted_send_post(user_gender, user_name))
        else
            local callback_request = framework.requests_processor:create_callback_request(data.request)
            if callback_request then
                cell_info.caption = get_post_description(callback_request:get_formatted_send_post(user_gender, user_name))
            end
        end
        cell_info.caption = get_post_description(data.request:get_formatted_receive_post(user_gender, user_name)) 
    end
end

function update_cell_callback_info(data)
    local user_name = social_api.get_normalized_user_name(data.profile, 0)
    local user_gender = sqc.framework.sqc_male
    if data.profile then
        user_gender = data.profile.gender
    end

    local cell_info
    if data.need_show_reward == true then
        cell_info = data.cell:find_child("cell_info_with_reward", true, false)
    else
        cell_info = data.cell:find_child("cell_info", true, false)
    end
    if cell_info then
        cell_info.visible = true
        if is_ask_response_thanks(data.info) and get_send_gift(data.request.friend_id) then
            cell_info.caption = get_post_description(data.request:get_formatted_send_post(user_gender, user_name))
        else
            local callback_request = framework.requests_processor:create_callback_request(data.request)
            if callback_request then
                cell_info.caption = get_post_description(callback_request:get_formatted_send_post(user_gender, user_name))
            end
        end
    end
end

function update_cell_profile(data)

    local user_name = social_api.get_normalized_user_name(data.profile, 0)
    local user_avatar = social_api.get_normalized_user_avatar(data.profile)
    local user_platform = social_api.get_normalized_user_platform(data.profile, true)
    local user_gender = sqc.framework.sqc_male

    if data.profile then
        user_gender = data.profile.gender
    end
    
    local cell_avatar = data.cell:find_child("cell_avatar", true, false)
    if cell_avatar then
        cell_avatar:set_seady_progress_as_image(social_api.get_default_avatar_name())
        if social_api.has_user_local_avatar(data.profile) then
            cell_avatar.texture_name = user_avatar
        else
            cell_avatar:set_remote_texture(user_avatar)
        end
    end

    local cell_name = data.cell:find_child("cell_name", true, false)
    if cell_name then
        cell_name.caption = user_name
    end

    if data.profile then
        local cell_exp = data.cell:find_child("cell_exp", true, false)
        if cell_exp then
            cell_exp.visible = true
        end

        local cell_exp_value = data.cell:find_child("cell_exp_value", true, false)
        if cell_exp_value then
            cell_exp_value.caption = tostring(data.profile.level)
        end

        local cell_platform = data.cell:find_child("cell_platform", true, false)
        if cell_platform then
            cell_platform.image_name = user_platform
        end
    end
end

function create_request_data(parent, request, rect)
    local cell = parent:create_child(rect)
    framework:json_ui_manager():build_from_resource("city_requests_cell", cell)

    local data = 
    {
        cell = cell,
        request = request,
        info = framework.requests_processor:get_request_info(request.name),
        profile = framework:friends():find_friend(request.friend_id, true),
        need_show_reward = false,
    }

    return data
end

function create_request_cell_impl(parent, request, rect)

    local data = create_request_data(parent, request, rect)

    update_cell_items(data)
    update_cell_info(data)
    update_cell_profile(data)
    update_cell_action(data)
    update_cell_cancel(data)
    update_cell_item_action(data)

    return data.cell
end

function create_request_cell_callback_impl(parent, request, rect)

    local data = create_request_data(parent, request, rect)

    update_cell_callback_items(data)
    update_cell_callback_info(data)
    update_cell_profile(data)
    update_cell_callback_action(data)
    update_cell_callback_cancel(data)
    update_cell_item_action(data)

    return data.cell
end

function create_request_cell(self, request, rect)
    local info = framework.requests_processor:get_request_info(request.name)

    if is_need_callback(request) then
        local cell = self.table:create_child(rect)
        local cell_accept = create_request_cell_impl(cell, request, rect)
        if cell_accept then
            cell_accept.tag = "cell_accept_state"
        end
        local cell_callback = create_request_cell_callback_impl(cell, request, rect)
        if cell_callback then
            cell_callback.tag = "cell_callback_state"
            cell_callback.visible = false
        end
        
        return cell
    elseif is_ask_response(request) then
        return create_request_cell_callback_impl(self.table, request, rect)
    end

    
    return create_request_cell_impl(self.table, request, rect) 
end

function create_request_category(category_info, parent)

    category_info.widget = framework:json_ui_manager():create_from_resource("city_requests_category", parent)
    local caption = category_info.widget:find_child("category_caption", true, false)
    if caption then
        local label = caption:find_child("category_label", true, false)
        if label then
            label.caption = string.format("%s (%d)", category_info.category.name, #category_info.requests)
        end
        caption.action = function() on_category_open(category_info) end
    end
    
    category_info.container = category_info.widget:find_child("category_container", true, false)
    if category_info.container then
        category_info.container.delegate = create_delegate(category_info)
    end

    return category_info.widget
end

function  get_max_category_scroll_cells()
    if g_device.is_tablet then
        return MAX_CATEGORY_SCROLL_CELLS_TABLE
    end
    return MAX_CATEGORY_SCROLL_CELLS
end

function create_content()
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        content = requests_table:create_child({0, 0, 0, 0})
        
        for i = 1, #category_infos do
            local info = category_infos[i]
            local category = create_request_category(info, content)

            

            if info.container then

                info.height = (info.container.cell_height + info.container.item_spacing) * math.min(#info.requests, get_max_category_scroll_cells())
                info.height_full = (info.container.cell_height + info.container.item_spacing) * #info.requests
           
           end
        end

        local old_height = requests_table.cell_height
        local old_scroll_pos = requests_table.scroll_pos[2]
        
        local change_pos = update_category_position(requests_table)

        requests_table:update_content()

        local new_scroll = old_scroll_pos
        if requests_table.cell_height < old_height then
            local delta = old_height - requests_table.cell_height
            print('requests_table delta:',delta)
            new_scroll = new_scroll - delta
        else
            new_scroll = new_scroll - change_pos
            print('requests_table change_pos:',change_pos)
        end
        requests_table.scroll_pos = {requests_table.scroll_pos[1],new_scroll}

        
    end
end

function update_category_position(table)
    local offset = 0
    local height = 0
    local change_pos = 0
    local rects = {}
    for i = 1, #category_infos do
        local info = category_infos[i]
        local category_rect = info.widget.rect
        if info.open then
            category_rect[4] = category_rect[4] + info.height
            change_pos = change_pos + update_category_content(info)
        end

        category_rect[2] = offset
        offset = offset + category_rect[4] + table.item_spacing
        rects[i]=category_rect
        
        height = category_rect[2] + category_rect[4]
    end
    for i = 1, #category_infos do
        local info = category_infos[i]
        local category_rect = rects[i]
        category_rect[2] = height - category_rect[2] - category_rect[4]
        update_category_rect(info, category_rect)
    end
    table.cell_height = height
    update_category_scroll(table.cell_height)
    return change_pos
end

function update_category_scroll(height)
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        local left = height
        local right = height

        for i = 1, #category_infos do
            right = left - get_category_caption_size()
            left = left - get_category_caption_size()
             
            if category_infos[i].open then
                right = left - category_infos[i].height 
                left = left - category_infos[i].height_full
            end

            if #category_infos[i].requests > get_max_category_scroll_cells() then
                category_infos[i].scroll.scrollable = true
            end

            category_infos[i].scroll.left = left
            category_infos[i].scroll.right = right
        end
    end
end

function update_category_rect(category_info, rect)
    local old_rect = category_info.widget.rect
    local offset = rect[4] - old_rect[4]

    category_info.widget.rect = rect
    local category_caption = category_info.widget:find_child("category_caption", true, fasle)
    if category_caption then
        local category_caption_rect = category_caption.rect
        category_caption_rect[2] = category_caption_rect[2] + offset
        category_caption.rect = category_caption_rect
    end
    if category_info.container then
        local category_container_rect = category_info.container.rect
        category_container_rect[4] = category_container_rect[4] + offset
        category_info.container.rect = category_container_rect
    end
end

function update_category_content(category_info)
    print("update_category_content")
    local result = 0
    if category_info.container and not(category_info.updated) then
        category_info.container:update_content()
        category_info.container:scroll_to_top()
        if category_info.scroll and category_info.scroll.pos and category_info.height_full then
            local old_height = category_info.old_full_height or 0

            -- have removed cell
            if old_height > category_info.height_full then 
                local delta = old_height - category_info.height_full
                print('delta:',delta)
                --- have scroll
                if category_info.container.scroll_area[4] > category_info.container.rect[4] then
                    category_info.scroll.pos = category_info.scroll.pos - delta
                    if category_info.scroll.pos < 0 then
                        result = -category_info.scroll.pos
                        print('result:',result)
                        category_info.scroll.pos = 0
                    end
                end
            end

            category_info.container.scroll_pos = {category_info.container.scroll_pos[1],
                category_info.scroll.pos}
        end
        category_info.updated = true
    end

    local category_up = category_info.widget:find_child("category_up", true, false)
    if category_up then
        category_up.visible = category_info.open
    end

    local category_down = category_info.widget:find_child("category_down", true, false)
    if category_down then
        category_down.visible = not(category_info.open)
    end

    return result
end

function lock_table(lock)
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        requests_table.disabled = lock
    end
end

-- category container delegate

function create_delegate(category_info)
    local delegate = 
    {
        info = category_info,
        create_cell = delegate_create_sell,
        get_cell_count = delegate_get_cell_count,
        allow_manual_scroll = delegate_allow_manual_scroll
    }
    return delegate
end

function delegate_create_sell(self, idx, rect)
    local cell=create_request_cell(self, self.info.requests[idx + 1], rect)
    cell.tag=cell.tag .. "_" .. idx
    return cell
end

function delegate_get_cell_count(self)
    return #self.info.requests
end

function delegate_allow_manual_scroll(self)
    return false
end

-- requests table delegate

requests_table_delegate = {}

function requests_table_delegate:create_cell(idx, rect)
    return content
end

function requests_table_delegate:get_cell_count()
    return 1
end

function requests_table_delegate:on_scroll() 
    scrolling(self.table, category_infos)
end

function scrolling(table, category_infos)
    local scroll_pos = table.scroll_pos[2]
    local pos = scroll_pos
    for i = 1, #category_infos do
        local info = category_infos[i]
        if info.container and info.open and info.scroll.scrollable then
            pos = pos - (info.container.scroll_area[4] - info.container.rect[4] - info.container.scroll_pos[2])
            if info.scroll.left < pos and pos < info.scroll.right then
                table.scroll_pos = {table.scroll_pos[1], info.widget.rect[2]}
                info.container.scroll_pos = {info.container.scroll_pos[1], info.container.scroll_pos[2] - (info.widget.rect[2] - scroll_pos)}
            elseif info.scroll.left > pos then
                info.container.scroll_pos = {info.container.scroll_pos[1], 0}
            elseif info.scroll.right < pos then
                info.container.scroll_pos = {info.container.scroll_pos[1], info.container.scroll_area[4] - info.container.rect[4]}
            end
        end
    end

end

-- actions

function on_category_open(info)
    print("[city_requests_dialog] open category action")
    if info.open then
        close_category(info)
    else
        open_category(info)
    end
end

function category_model(category_info)
    local model = {}
    model.rects = {}
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        model.cell_height = requests_table.cell_height
        model.scroll_pos = requests_table.scroll_pos
    end
    for i = 1, #category_infos do
        table.insert(model.rects, category_infos[i].widget.rect)
    end
    return model
end

function open_category(category_info)
    category_info.open = true
    category_animation_params = {}
    category_animation_params.time = math.min(#category_infos * 200, 400)
    if not(category_info.updated) then
        category_animation_params.time = 0
    end
    category_animation_params.begin_value = 0.0
    category_animation_params.end_value = category_info.height
    category_animation_params.info = category_info
    category_animation_params.model = category_model(category_info)
    lock_table(true)
end

function close_category(category_info)
    category_info.open = false
    category_animation_params = {}
    category_animation_params.time = math.min(#category_infos * 200, 400)
    category_animation_params.begin_value = 0.0
    category_animation_params.end_value = -category_info.height
    category_animation_params.info = category_info
    category_animation_params.model = category_model(category_info)
    lock_table(true)
end

function is_category_animation()
    return not(category_animation_params == nil)
end

function category_animate(tick)
    if is_category_animation() then
        if not(category_animation_params.tick) then
            category_animation_params.tick = tick
        end
        local delta = (tick - category_animation_params.tick)
        if delta < category_animation_params.time then
            local k = delta / category_animation_params.time
            local value = category_animation_params.end_value * k + category_animation_params.begin_value * (1.0 - k)
            category_animation_pulse(value, 
                                     category_animation_params.info, 
                                     category_animation_params.model)
        else
            category_animation_pulse(category_animation_params.end_value, 
                                     category_animation_params.info, 
                                     category_animation_params.model)
            update_category_content(category_animation_params.info)
            category_animation_params = nil
            lock_table(false)
        end
    end
end

function category_animation_pulse(value, category_info, model)
    value = math.ceil(value)
    local requests_table = dialog:find_child("requests_table", true, false)
    if requests_table then
        requests_table.cell_height = model.cell_height + value
        requests_table.scroll_pos = {model.scroll_pos[1], model.scroll_pos[2] + value}
    end

    for i = 1, #category_infos do
        if category_info.category.tag == category_infos[i].category.tag then
            local rect = category_infos[i].widget.rect
            rect[4] = model.rects[i][4] + value
            update_category_rect(category_infos[i], rect)
            break
        end
        local rect = category_infos[i].widget.rect
        rect[2] = model.rects[i][2] + value
        category_infos[i].widget.rect = rect
    end

    update_category_scroll(requests_table.cell_height)
end

function show_barn_update_animation(data, close_window, lock_dialog)
    
    local type_names={}
    local reward_rects={}

    table.sort(data.requests_data, function(a, b) return  a.time < b.time end)
    
    
    for idx, request in ipairs(data.requests_data) do

        local type_id=request.item_id
        local item_info = framework:object_factory():get_object_info_by_type_id(type_id)
        
        table.insert(type_names, item_info.type_name)
    end   

    if data.cell then

       local cell_item = data.cell:find_child("cell_right_item_bg", true, false)

        if cell_item then
            table.insert(reward_rects,cell_item.screen_rect)
        end
    end

    
    
    if #type_names > 0 and barn_animator then
        if barn_animator_params then
            barn_animator_params.close_window=close_window
            barn_animator_params.disable_dialog=lock_dialog
        end
        barn_animator:set_rewards_data(type_names,reward_rects)
        barn_animator:show_barn_update_animation(barn_animator_params)
    end
    
end

function accept_request(data)
    lock_interface(true)
    lock_table(false)
    
    cancel_accept_request_animation()

    if is_ask_response(data.request) and is_ask_response_thanks(data.info) then
        reset_send_gift_cache(data.request.friend_id)
    end

    disable_barn_animation=false

    --need to check it here because it may be changed in  requests_controller:accept_request
    if is_ask_response(data.request) then
       disable_barn_animation=true
    end

    requests_controller:accept_request(data.request)
    
    if (data.info.accept_action == sqc.requests.sqc_request_accept_action.request_accept_action_visit_farm or
        data.info.accept_action == sqc.requests.sqc_request_accept_action.request_accept_action_visit_object) then
        dialog:end_dialog()
    else
        lock_interface(false)
        local requests_count=requests_controller:get_request_count()
        local close_dialog=false
        local lock_dialog=false

        if requests_count ==1 and not is_ask_response(data.request) then
            close_dialog=true
        end
        
        --if data.request.name=="send_gift_new" and not is_need_callback(data.request) then
        --    show_barn_update_animation({cell=data.cell,requests_data={data.request}}, close_dialog, lock_dialog)
        --end

        if ((data.request.name=="send_gift_new") and  not disable_barn_animation) then
            show_barn_update_animation({cell=data.cell,requests_data={data.request}}, close_dialog, lock_dialog)
        end
    end
   
end

function use_request_object(data)
    lock_interface(true)
    lock_table(false)

	cancel_accept_request_animation()
	
	requests_controller:use_request_object(data.request)
	
	if is_buildable_gift(data) == true and is_storable_gift(data) then
		dialog:end_dialog()
	else
		lock_interface(false)	
	end
end

function animate_accept_request(data)
    local parent = data.cell:get_parent()
    local current_state = parent:find_child("cell_accept_state", true, false)
    local next_state = parent:find_child("cell_callback_state", true, false)

    if current_state and next_state then
        next_state.visible = true

        lock_table(true)
        request_animate_params = 
        {
            current_transform = current_state:transform(),
            next_transform = next_state:transform()
        }
        framework:windows():animation():add_animator(request_animate_params.next_transform, 
            cclasses.animation_animator_sequency.new(false)
                :add(cclasses.animation_animator_alpha.new(0, 1, 1)))

        framework:windows():animation():add_animator(request_animate_params.current_transform, 
            cclasses.animation_animator_sequency.new(false)
                :add(cclasses.animation_animator_alpha.new(1, 0, 0.7))
                :add(cclasses.animation_animator_event.new(function() current_state.visible = false end)))
    end
end

function on_cell_accept_action(data)
    print("[city_requests_dialog] accept request action")
    
    if barn_animator then
        barn_animator.animation_started=false
    end

    if is_need_callback(data.request) then
        animate_accept_request(data)
    end

    accept_request(data)
    
end


function on_cell_use_request_object(data)
    print("[on_cell_use_request_object] use_request_object")

    if barn_animator then
        barn_animator.animation_started=false
    end

    use_request_object(data)
end


function on_cell_accept_unactive_action(data)
    print("[city_requests_dialog] accept request unactive action")
    if check_can_accept_today(data.info) then
        local error = data.request.error_description
        if city_notenough_dialogs.error_can_be_shown(error) then
            city_notenough_dialogs.show_not_enough_for_shop_error(error)
        else
            local params = {}
            params.type = 1
            params.dlg_text = framework:shop():get_error_description(error);
            params.dlg_title = framework:strings():load("*CityBuyError","Error");
            params.button_text = framework:strings():load("*CityOKButton","OK");
            framework:dialogs():show_message_box(params);
        end
    else
        show_tomorow_dlg()
    end
end

function on_cell_cancel_action(data)
    print("[city_requests_dialog] cancel request action")
    
    if barn_animator then
        barn_animator.animation_started=false
    end

    lock_interface(true)
    requests_controller:cancel_request(data.request)
    lock_interface(false)

end

function lock_interface(lock)
    if lock and not(reqeusts_lock) then
        reqeusts_lock = true
        dialog:lock_interface()
    else
        reqeusts_lock = false
    end
end

function is_lock_interface()
    return reqeusts_lock
end

function on_timer(tick)
    if dialog then
        if is_category_animation() then
            category_animate(tick)
        end
    end
end




function on_requests_accept_all_button_action()
    print("[city_requests_dialog] accept all requests action.")
    
    
    local requests = {}
    for i = 1, requests_controller:get_request_count() do
        local request = requests_controller:get_request(i - 1)
        local request_info = framework.requests_processor:get_request_info(request.name)
        if check_auto_accept_allowed(request,request_info) == sqc.requests.sqc_request_availability.sqc_request_available then
        	requests[#requests+1] = request
       	end        	
    end
    
    lock_interface(true)
    framework:windows():animation():add_animator(dialog:transform(), 
        cclasses.animation_animator_sequency.new(false)
        :add(cclasses.animation_animator_pause.new(1))
        :add(cclasses.animation_animator_event.new(function() 
            requests_controller:auto_accept_requests(requests)
            lock_interface(false) 
            end)))


    show_barn_update_animation({cell=nil , requests_data=requests}, true, true)

end

function on_requests_accept_all_button_unactive_action()
    print("[city_requests_dialog] accept all requests unactive action")
    if not (answers_available == 0) then
        show_not_availabel_auto_accept_requests()
    else
        show_tomorow_dlg()
    end
end

function show_tomorow_dlg()
    local params = {}
    params.type = 0
    params.dlg_text = framework:strings():load("*RequestEmptyAvailabel", "Tomorrow you'll be able to answer next 30 letters.");
    params.dlg_title = framework:strings():load("*CommonBuyError","Error!");
    params.button_text = framework:strings():load("*CityOKButton","OK");
    framework:dialogs():show_message_box(params);
end

function show_not_availabel_auto_accept_requests()
    local params = {}
    params.type = 0
    params.dlg_text = framework:strings():load("*CityRequestNotAvailableAutoAcceptRequests", "You have not requests to automatically accept");
    params.dlg_title = framework:strings():load("*CityRequestAskItemMessageBoxCaption","Information");
    params.button_text = framework:strings():load("*CityOKButton","OK");
    framework:dialogs():show_message_box(params);
end

function on_requests_room_specific_action()
    local room_specific_dialog = cclasses.sqc_scriptable_dialog.create(framework:windows().desktop, "city_requests_room_specific_dialog.lua")
    if room_specific_dialog then  
        room_specific_dialog.scripting_object.room_specific_requests = room_specific_requests
        room_specific_dialog:do_modal()
    end
end


