
local room_infos = {}

function on_init_dialog()
    dialog:load_content("city_requests_room_specific_dialog")

    update_room_infos()
    update_dialog_size()

    local requests_table = dialog:find_child("requests_room_specific_table", true, false)
    if requests_table then
        requests_table:update_content()
    end
end

function on_post_init_dialog()
end

function end_dialog()
end

function on_close_button_pressed()
    dialog:end_dialog()
end

function update_room_infos()
	local room_id = framework:game().room_id
	local rooms = rooms_api.get_rooms_info()
	for i = 1, #rooms do
		if not(rooms[i].id == room_id) and rooms[i].opened then
			table.insert(room_infos, rooms[i])
		end
	end
end

function get_cell_height()
	if g_device.is_tablet then
		return 92
	end
	return 66
end

function get_max_cell_count()
	if g_device.is_tablet then
		return 4
	end
	return 4
end

function update_dialog_size()
	local count = math.min(#room_infos, get_max_cell_count())
	local height = get_cell_height() * (count - 1)
	local rect = dialog.rect
	rect[4] = rect[4] + height
	dialog.rect = rect

	local requests_table = dialog:find_child("requests_room_specific_table", true, false)
    if requests_table then
        local rect_table = requests_table.rect
        rect_table[4] = rect_table[4] + height
        requests_table.rect = rect_table
    end

    local requests_title = dialog:find_child("requests_room_specific_title", true, false)
    if requests_title then
    	local rect_title = requests_title.rect
        rect_title[2] = rect_title[2] + height
        requests_title.rect = rect_title
    end   
end

-- room specific table delegate

requests_room_specific_table_delegate = {}

function requests_room_specific_table_delegate:create_cell(idx, rect)
	local cell = self.table:create_child(rect)
    framework:json_ui_manager():build_from_resource("city_requests_room_specific_cell", cell)
   
    local index = idx + 1
    local cell_title = cell:find_child("cell_title", true, false)
    if cell_title then
    	cell_title.caption = room_infos[index].name
    end

    local letter_count = 0
    for i = 1, #room_specific_requests do
    	if room_specific_requests[i].room_id == room_infos[index].id then
    		letter_count = letter_count + 1
    	end
    end

    local genitive_forms = framework:strings():load("*MessageFirstForm", "message") .. "\n" ..
    					   framework:strings():load("*MessageSecondForm", "messages") .. "\n" .. 
    					   framework:strings():load("*MessageThirdForm", "messages") .. "\n";

    local letter_count_title = string.format("%d %s", letter_count, framework:formats():format_plural(letter_count, genitive_forms))
    local letter_count_label = cell:find_child("cell_letter_count", true, false)
    if letter_count_label then
    	letter_count_label.caption = letter_count_title
    end

    local letter_go_button = cell:find_child("cell_letter_go", true, false)
    if letter_go_button then
    	if letter_count == 0 then
    		letter_go_button.active = false
    		letter_go_button.unactive_action = function() on_goto_room_unactive_action(index) end
    	end
    	letter_go_button.action = function() on_goto_room_action(index) end
    end

    return cell
end

function requests_room_specific_table_delegate:get_cell_count()
	return #room_infos
end

function on_goto_room_unactive_action(index)
	print("[city_requests_room_specific_dialog] go to unactive action")
	local params = {}
    params.type = 0
    params.dlg_text = framework:strings():load("*RequestGotoRoomError", "You have no letters available");
    params.dlg_title = room_infos[index].name
    params.button_text = framework:strings():load("*CityOKButton","OK");
    framework:dialogs():show_message_box(params);
end

function on_goto_room_action(index)
	print("[city_requests_room_specific_dialog] go to action")
	rooms_api.goto_room(room_infos[index].id, 0, on_got_start, on_goto_complete, on_goto_error, false)
end

function on_got_start()
end

function on_goto_complete()
	print("[city_requests_room_specific_dialog] go to complete")
	dialog:end_dialog()
end

function on_goto_error()
end





