local months_names = framework:get_client_string("months_names")
local m_i_last_ticked_periods_count=0
local competition_history_cells = {}

function update_content()
   -- When the build is greater than 2472 the spinboard delegate is
   -- set earlier than the spinboard rect is set to correct value.
   -- Therefore we have wrong cells width.
   if g_application.build > 2472 then
      dialog:find_child('competitions_scroll'):update_content()
   end
end

function on_init_dialog()
	print('on_init_dialog')
	dialog:load_content('competition_history')

    if competition_history_delegate then
        competition_history_controller = competition_history_delegate
    end

	competition_history = competition_history_controller:get_competition_history()	

	update_content()

	local scroll_up = dialog:find_child('competitions_scroll',true,false)
	if scroll_up then
		scroll_up.scroll_pos = {0, math.max(0,(scroll_up.cell_height + scroll_up.item_spacing) * #competition_history - scroll_up.rect[4] + scroll_up.bottom_fade) }
	end

	local competition_history_caption = dialog:find_child('competition_history_caption',true,false)
	if competition_history_caption then
		competition_history_caption.caption = framework:get_client_string("competitions_archive")
	end
end

function on_post_init_dialog()
    local back_button = dialog:find_child( "back_button", true, false )
    if back_button then
        back_button.action =
            function( tick )
		on_back_button_action()
            end
    end
end

function on_back_button_action()
    print ("on_back_button_action()")
    if competition_history_controller:on_back_button_pressed() then
        dialog:end_dialog()
    end
end

competitions_scroll_delegate = {}

function competitions_scroll_delegate:create_cell(idx,rect)
	local w = self.table:create_child(rect)
	competition_history_cells[idx+1] = w
	framework:json_ui_manager():build_from_resource("competition_history_cell", w)
	local competition_name = competition_history[idx+1].event_name
	local lable_competition_name = competition_history_cells[idx+1]:find_child('competition_name',true,false)		
	if lable_competition_name then 
		lable_competition_name.caption = framework:get_client_string(competition_name .. ":short_name")
	end
	
	local image_competition_icon = w:find_child('competition_icon',true,false)
	if image_competition_icon then 
		if g_device.is_tablet then      -- TABLET
			image_competition_icon.texture_name = competition_name .. "_ipad"
		else                            -- PHONE
			image_competition_icon.texture_name = competition_name
		end
	end

	function on_goto_open_competition()
		dialog:end_dialog()
		competition_history[idx+1]:on_action()
	end

	update_time_related_widgets()

	local btn_open_competition = w:find_child("button", true, false);
	if btn_open_competition then
        btn_open_competition.action = on_goto_open_competition
    end
	return w
end

function update_time_related_widgets()
	local minimum_index = math.min (#competition_history_cells, #competition_history)
	for i=1, minimum_index do 
		local calculated = competition_history[i].calculated
		local competition_timer = competition_history_cells[i]:find_child('timer_date_label',true,false)
		local competition_time = competition_history[i]:get_time_remaining_sec()
		local competition_timer_visible = competition_history_cells[i]:find_child('timer_date_bg',true,false)
		local competition_calculation_visible = competition_history_cells[i]:find_child('competition_calculation',true,false)
		local cell_bg_competition = competition_history_cells[i]:find_child('bg_cell_competition',true,false)
		local cell_competition_status = competition_history_cells[i]:find_child('competition_status',true,false)
		local cell_competition_status_lable = competition_history_cells[i]:find_child("cell_competition_status_lable", true, false)
		local competition_start_begin_date = competition_history[i].start_begin_date
		local competition_start_end_date = competition_history[i].start_end_date
		local competition_year_begin_date = competition_history_controller:number_of_year(competition_start_begin_date)
		local competition_month_begin_date = competition_history_controller:number_of_month(competition_start_begin_date)
		local competition_year_end_date = competition_history_controller:number_of_year(competition_start_end_date)
		local competition_month_end_date = competition_history_controller:number_of_month(competition_start_end_date)
		
		if calculated or competition_time < 0 then
	
			if competition_timer_visible then
				competition_timer_visible.visible = false
			end	
			
			if competition_calculation_visible then
				competition_calculation_visible.visible = not calculated
			end
			
			if cell_bg_competition then
				if g_device.is_tablet then      -- TABLET
					cell_bg_competition.image_name = "competition_history_item_blue_ipad"
				else                            -- PHONE
					cell_bg_competition.image_name = "competition_history_item_blue"
				end
			end
			
			if cell_competition_status then
				cell_competition_status.visible = true
				if cell_competition_status_lable then 
					cell_competition_status_lable.visible = false
				end
				cell_competition_status_text = framework:get_client_string("competition_archive_window:competition_was_conducted_mobile")
				local months_name_copy = months_names
				local months_name_second_copy = months_names
                
				local str_month_begin = ""
				for i=1,competition_month_begin_date do
					local i_moment1_found=string.find(months_name_copy, "\n", 1, true)
					if i_moment1_found ~= nill then
						str_month_begin = string.sub(months_name_copy, 1, i_moment1_found-1)
						months_name_copy=string.sub(months_name_copy, i_moment1_found+1, -1)
					end
				end

                local str_month_end = ""
				for i=1,competition_month_end_date do
					local i_moment1_found=string.find(months_name_second_copy, "\n", 1, true)
					if i_moment1_found ~= nill then
						str_month_end = string.sub(months_name_second_copy, 1, i_moment1_found-1)
						months_name_second_copy=string.sub(months_name_second_copy, i_moment1_found+1, -1)
					end
				end
				
				if competition_year_begin_date == competition_year_end_date then
					if 	competition_month_begin_date == competition_month_end_date then
						competition_dates = str_month_begin .. " " .. competition_year_end_date
					else
						competition_dates = str_month_begin .. " - " .. str_month_end .. "\n" .. competition_year_begin_date
					end	
				else 
					competition_dates = str_month_begin .. " " .. competition_year_begin_date .. " -\n" .. str_month_end .. " " .. competition_year_end_date
				end
				
				cell_competition_status.caption = string.gsub(cell_competition_status_text, "{.?}", competition_dates)
			end
			
			cell_button_text = competition_history_cells[i]:find_child('button_text',true,false)
			if cell_button_text then
				cell_button_text.caption = framework:get_client_string("competition_archive_window:go_to_top")
			end
		else
			if competition_timer_visible then
				competition_timer_visible.visible = competition_time > 0
			end

			if competition_calculation_visible then
				competition_calculation_visible.visible = competition_time < 0
			end

			if competition_timer then
				competition_timer.time = competition_time
			end

			if cell_bg_competition then
				if g_device.is_tablet then      -- TABLET
					cell_bg_competition.image_name = "competition_history_item_yellow_ipad"
					font_size = 20
					status_rect_x = 362
					status_rect_width = 145
				else                            -- PHONE
					cell_bg_competition.image_name = "competition_history_item_yellow"
					font_size = 14
					status_rect_x = 241
					status_rect_width = 98
				end
			end		
	
			if cell_competition_status then
				cell_competition_status.visible = false
				if not cell_competition_status_lable then 				
					local lable_active_competition_json = "{\"type\": \"milk_label\", \"outline_type\": \"double\", \"outline_soft\": true, \"rect\": [ ".. status_rect_x - 5 .. ", 2, ".. status_rect_width ..", 54], \"color\": [201, 104, 0, 255 ], \"text_halign\": \"center\",\"text_valign\": \"center\",\"valign\": \"center\",\"text_adjust_font_size\": true,\"shadow_color\": [255, 255, 255, 255],\"tag\": \"competition_status\",\"text_shadow\": \"true\",\"text_line_spacing\": \"80%\",\"font\": { \"face\": \"Arial-BoldMT.otf\", \"size\": " .. font_size .. "}}"
					cell_competition_status_lable = framework:json_ui_manager():create_from_string(lable_active_competition_json, "cell_competition_status_lable", cell_bg_competition)		
					cell_competition_status_lable.caption = framework:get_client_string("competition_archive_window:competition_conducted")
				else
					cell_competition_status_lable.visible = true
				end
			end
			cell_button_text = competition_history_cells[i]:find_child('button_text',true,false)
			if cell_button_text then
				cell_button_text.caption = framework:get_client_string("competition_archive_window:go_to_competition")
			end

		end
	end
end

function on_timer(tick)
	local i_ticked_periods_delta = math.floor(tick/1000) - m_i_last_ticked_periods_count
	if i_ticked_periods_delta > 0 then
		update_time_related_widgets()	
		m_i_last_ticked_periods_count=m_i_last_ticked_periods_count+i_ticked_periods_delta
	end
end

function competitions_scroll_delegate:get_cell_count()
	return #competition_history
end
