function on_init_dialog( )
    dialog.theme_flags = 0
    dialog.bevel_type = 0
    dialog.draw_arg_type = 1

    if contract_info_controller then
        m_controller = contract_info_controller
    end

    dialog:load_content('contract_info_dlg')

    if g_device.is_tablet then
        cell_height = 67
        cell_offset = 3
    else
        cell_height = 38
        cell_offset = 2
    end

    local cell_count = m_controller:get_require_object_count()

    local y = cell_height * cell_count + cell_offset * (cell_count - 1)
    
    local r = dialog.rect
    r[4] = r[4] + y
    dialog.rect = r

    local area = dialog:create_child()
    framework:json_ui_manager():build_from_resource("contract_info_dlg_area", area)

    local contract_name_label = area:find_child("contract_info_name", true, false)
    if contract_name_label then
        contract_name_label.caption = m_controller:get_contract_description()
    end

    local total_buy_count = 0
    for index = 0, cell_count - 1 do
       local cur_count = m_controller:get_require_object_count_on_field(index)
       local material_count = m_controller:get_require_object_material_count(index)
       local buy_count = (material_count > 0) and material_count or cur_count
       total_buy_count = total_buy_count + buy_count
    end

    local explanation_text_label = area:find_child("explanation_text_label", true, false)
    if explanation_text_label then
       if total_buy_count > 1 then
          explanation_text_label.caption = framework:strings():load("*ContractInfoExplanationText", "Construction of additional objects increases the cost and rewards for contract")
       else
          explanation_text_label.caption = framework:strings():load("*ContractInfoExplanationTextWithoutCost", "Construction of additional objects increases rewards for contract")
       end
    end

    local total_area = area:find_child("contract_info_total_area", true, false)
    if total_area then
        local total_area_rect = total_area.rect;
        total_area_rect[2] = total_area_rect[2] - y
        total_area_rect[4] = total_area_rect[4] + y
        total_area.rect = total_area_rect

        local separator = total_area:find_child("contract_info_total_area_separator", true, false)
        if separator then
            local  separator_rect = separator.rect
            separator_rect[4] = separator_rect[4] + y
            separator.rect = separator_rect
        end

        local cost_title = total_area:find_child("contract_info_cost_caption", true, false)
        if cost_title then
            local cost_title_rect = cost_title.rect
            cost_title_rect[2] = cost_title_rect[2] + y
            cost_title.rect = cost_title_rect
        end

        local reward_title = total_area:find_child("contract_info_reward_caption", true, false)
        if reward_title then
            local reward_title_rect = reward_title.rect
            reward_title_rect[2] = reward_title_rect[2] + y
            reward_title.rect = reward_title_rect
        end
    end

    total_cost = 0
    total_reward = 0

    for i = 0, cell_count - 1 do 
        create_new_cell(dialog, i)
    end

    local total_cost_label = total_area:find_child("contract_info_total_cost", true, false)
    if total_cost_label then
        total_cost_label.caption = framework:formats():format_digit_grouping(total_cost)
    end

    local total_cost_coins_icon_stub = total_area:find_child("total_cost_coins_icon_stub", true, false)
    if total_cost_coins_icon_stub then
      m_controller:create_coins_buy_icon(0, total_cost_coins_icon_stub);
    end

    local total_reward_label = total_area:find_child("contract_info_total_reward", true, false)
    if total_reward_label then
        total_reward_label.caption = "+" .. framework:formats():format_digit_grouping(total_reward)
    end

    local total_reward_coins_icon_stub = total_area:find_child("total_reward_coins_icon_stub", true, false)
    if total_reward_coins_icon_stub then
      m_controller:create_coins_sell_icon(0, total_reward_coins_icon_stub);
    end

    local m_ok_btn = dialog:find_child("ok_btn", true, false)
    if m_ok_btn then
        m_ok_btn.action = on_ok_btn
    end

    local m_close_btn = dialog:find_child("close_btn", true, false)
    if m_close_btn then
        m_close_btn.action = on_ok_btn
    end
end

function on_post_init_dialog( )
end

function end_dialog( )
end

function on_ok_btn()
    dialog:end_dialog()
end

function on_add_btn(index)
    if (m_controller:get_require_object_max_count(index) > m_controller:get_require_object_count_on_field(index)) then
	dialog:end_dialog()
    end
    m_controller:on_add_object(index)
end

function create_new_cell(parent, index)
    local cell = parent:create_child()
    framework:json_ui_manager():build_from_resource("contract_info_object_cell", cell)

    local r = cell.rect
    r[2] = r[2] - (cell_height + cell_offset) * index
    cell.rect = r

    local descr = cell:find_child("contract_object_name", true, false)
    if descr then
        descr.caption = m_controller:get_require_object_description(index);
    end

    local image = cell:find_child("contract_object_image", true, false)
    if image then
        image.texture_name = m_controller:get_require_object_image(index)
        image:set_ready_progress_as_indicator()
    end 

    local cur_count = m_controller:get_require_object_count_on_field(index)
    local max_count = m_controller:get_require_object_max_count(index)

    local cur_count_label = cell:find_child("contract_object_current_count", true, false)
    if cur_count_label then
        cur_count_label.caption = tostring(cur_count)
    end

    local max_count_label = cell:find_child("contract_object_max_count", true, false)
    if max_count_label then
        max_count_label.caption = tostring(max_count)
    end

    local cost = m_controller:get_require_object_buy(index)

    local material_count = m_controller:get_require_object_material_count(index)
    local buy_count = (material_count > 0) and material_count or cur_count

    local cost_label = cell:find_child("contract_object_cost", true, false)
    if cost_label then
        cost_label.caption = "+" .. framework:formats():format_digit_grouping(cost) .. " x " .. tostring(buy_count)
    end

    total_cost = total_cost + cost * buy_count

    local cost_coins_icon_stub = cell:find_child("contract_object_cost_coins_icon_stub", true, false)
    if cost_coins_icon_stub then
      m_controller:create_coins_buy_icon(index, cost_coins_icon_stub);
    end

    local reward = m_controller:get_require_object_sell(index)
    local reward_label = cell:find_child("contract_object_reward", true, false)
    if reward_label then
        reward_label.caption = "+" .. framework:formats():format_digit_grouping(reward) .. " x " .. tostring(cur_count)
    end
    total_reward = total_reward + reward * cur_count

    local reward_coins_icon_stub = cell:find_child("contract_object_reward_coins_icon_stub", true, false)
    if reward_coins_icon_stub then
      m_controller:create_coins_sell_icon(index, reward_coins_icon_stub);
    end

    local add_btn = cell:find_child("add_btn", true, false)
    if add_btn then
        add_btn.action = function()
            on_add_btn(index)
        end
        add_btn.unactive_action = function()
            on_add_btn(index)
        end
        if m_controller:get_require_object_count_on_field(index) >= m_controller:get_require_object_max_count(index) then
            add_btn.active = false
        end
    end
end
