local num_tick = 0

local offers = {}
local timers = {}

function on_init_dialog()
    dialog:load_content("cross_promo_dialog")
    update_dialog()
end

function on_post_init_dialog()
    adjust_title_dialog()
end

function adjust_title_dialog()
    local dialog_title_caption = dialog:find_child('dialog_title_caption', true, false)
    if dialog_title_caption then
        local caption_rect = dialog_title_caption.rect
        if g_device.is_tablet then
            caption_rect[1] = 40
            caption_rect[3] = 560
        elseif g_device.is_phone then
            caption_rect[1] = 22
            caption_rect[3] = 395
        end
        dialog_title_caption.rect = caption_rect
        dialog_title_caption.text_adjust_font_size = true
    end

    local dialog_title_bevel = dialog:find_child('dialog_title_bevel', true, false)
    if dialog_title_bevel then
        local title_bevel_rect = dialog_title_bevel.rect
        if g_device.is_tablet then
            title_bevel_rect[1] = 30
            title_bevel_rect[3] = 580
        elseif g_device.is_phone then
            title_bevel_rect[1] = 15
            title_bevel_rect[3] = 410
        end
        dialog_title_bevel.rect = title_bevel_rect
    end

    local cross_promo = framework:game():get_cross_promo_manager()
    if not cross_promo then
        print("[cross_promo_dialog] cross promo manager hasn't been initialized")
        return
    end

    local dialog_title_caption = dialog:find_child('dialog_title_caption', true, false)
    if dialog_title_caption then
        local title_caption = cross_promo.title_caption
        if title_caption ~= "" then
            dialog_title_caption.caption = title_caption
        end
    end
end

function on_timer(tick)
    if not dialog then
	return
    end

    if tick > num_tick then
        num_tick = tick + 500

        local cross_promo = framework:game():get_cross_promo_manager()
        if not cross_promo then
            print("[cross_promo_dialog] cross promo manager hasn't been initialized")
            return
        end

        if #offers == 0 or #offers ~= #timers then
            return
        end

        for i = 1, #offers do
            local time_left = cross_promo:get_calculated_time_left_for_offer( offers[i].offer_id )
            if time_left >= 0 then
                timers[i].time = time_left
            end
        end

        local offers_count = #cross_promo:get_not_accepted_offers() + #cross_promo:get_accepted_offers()
        if offers_count ~= #offers then
            if offers_count == 0 then
                dialog:end_dialog()
                return
            end

            update_dialog()
            return
        end
    end
end

function update_dialog()
    offers = {}
    timers = {}

    update_offers()

    local offers_table = dialog:find_child("offers_table", true, false)
    if offers_table then
        offers_table:update_content()
        offers_table:scroll_to_top()
    end
end

function update_offers()
    local cross_promo = framework:game():get_cross_promo_manager()
    if not cross_promo then
        print("[cross_promo_dialog] cross promo manager hasn't been initialized")
        return
    end

    local not_accepted_offers = cross_promo:get_not_accepted_offers()
    for i = 1, #not_accepted_offers do
        table.insert(offers, not_accepted_offers[i])
    end

    local accepted_offers = cross_promo:get_accepted_offers()
    for i = 1, #accepted_offers do
        table.insert(offers, accepted_offers[i])
    end

    -- print("[cross_promo_dialog] offers")
    -- for i = 1, #offers do
    --     local offer = offers[i]
    --     print( "  offer with id " .. offer.offer_id .. ":" )
    --     local params_map = offer:get_offer_params_map()
    --     for key, val in pairs(params_map) do
    --         print( "    [" .. key .. "] " .. val )
    --     end
    -- end
    -- print("---------------------------")
end

function enable_cell(cell, enable)
    if not cell then return end
    local button = cell:find_child("cell_button_accepted", true, false)
    if button then 
        button.active = enable
    end
    button = cell:find_child("cell_button_created", true, false)
    if button then 
        button.active = enable
    end
    local cell_bg = cell:find_child("cell_bg", true, false)
    if cell_bg then
        cell_bg.active = enable
    end
end

function update_cell(cell, offer)
    local cell_button
    local cell_border
    local cell_action

    if not offer or not cell then
        print("[cross_promo_dialog] object is null")
        return
    end

    enable_cell(cell, true)

    if offer.offer_state == sqc.sqc_offers.sqc_offer_state.sqc_offer_state_created then
        cell_button = cell:find_child("cell_button_created", true, false)
        cell_border = cell:find_child("cell_border_created", true, false)
        cell_action = function()
            on_button_created_action(offer, cell)
        end
    elseif offer.offer_state == sqc.sqc_offers.sqc_offer_state.sqc_offer_state_accepted then

        local button = cell:find_child("cell_border_created", true, false)
        if button then
           button:destroy()
        end

        cell_button = cell:find_child("cell_button_accepted", true, false)
        cell_border = cell:find_child("cell_border_accepted", true, false)
        cell_action = function()
            on_button_accepted_action(offer, cell)
        end
    else
        print("[cross_promo_dialog] offer status isn't created or accepted")
        return nil
    end

    if cell_button and cell_border and cell_action then
        cell_button:set_visible(true)
        cell_border:set_visible(true)
        cell_button.action = cell_action
    else
        print("[cross_promo_dialog] can't find button and border on cell")
    end

    local cell_bg = cell:find_child("cell_bg", true, false)
    if cell_bg and cell_action then
        cell_bg.action = cell_action
    else
        print("[cross_promo_dialog] can't find background on cell")
    end
end

local update_dlg_callback = function()
    print("[cross_promo_dialog] update_dlg_callback")

    if not dialog then
	return
    end

    local offers_table = dialog:find_child("offers_table", true, false)
    if not offers_table then
        return
    end

    local cells = offers_table:get_cells()
    for i = 1, #cells do
        update_cell(cells[i], offers[i])
    end
end

function on_button_created_action(offer, cell)
    print("[cross_promo_dialog] on button created action")

    if offer then
        local cross_promo = framework:game():get_cross_promo_manager()
        if not cross_promo then
            print("[cross_promo_dialog] cross promo manager hasn't been initialized")
            return
        end

        enable_cell(cell, false)

        cross_promo:set_update_dialog_callabck(update_dlg_callback)
        cross_promo:accept_offer( offer.offer_id )
    end
end

function on_button_accepted_action(offer, cell)
    print("[cross_promo_dialog] on button accepted action")

    if offer and offer.offer_state == sqc.sqc_offers.sqc_offer_state.sqc_offer_state_accepted then
        framework:platform():open_url(offer.start_link)
    end
end

function on_end_dialog()
    print("[cross_promo_dialog] on end dialog")

    local cross_promo = framework:game():get_cross_promo_manager()
    if cross_promo then
        cross_promo:set_update_dialog_callabck(nil)
    end

    dialog:end_dialog()
end


-- offers table delegate

offers_table_delegate = {}

function offers_table_delegate:create_cell(idx, rect)
    if #offers == 0 then
        print("[cross_promo_dialog] offers size is 0")
        return nil
    end

    local index = idx + 1
    local offer = offers[index]

    if not offer then
        print("[cross_promo_dialog] offer object is null")
        return nil
    end

    local cross_promo = framework:game():get_cross_promo_manager()
    if not cross_promo then
        print("[cross_promo_dialog] cross promo manager hasn't been initialized")
        return nil
    end

    local cell = self.table:create_child(rect)
    framework:json_ui_manager():build_from_resource("cross_promo_cell", cell)

    local cell_image = cell:find_child("cell_image", true, false)
    if cell_image then
        cell_image:set_remote_texture(offer.icon_name)
        cell_image:set_seady_progress_as_image("quest0");
    end

    local cell_title = cell:find_child("cell_title", true, false)
    if cell_title then
        cell_title.caption = offer.title
    end

    local cell_description = cell:find_child("cell_description", true, false)
    if cell_description then
        cell_description.caption = offer.description
    end

    local cell_timer_image = cell:find_child("cell_timer_image", true, false)
    local cell_timer_label = cell:find_child("cell_timer_label", true, false)
    if cell_timer_image and cell_timer_label then
        local time_left = cross_promo:get_calculated_time_left_for_offer( offer.offer_id )
        if time_left > 0 then
            cell_timer_image:set_visible(true)
            cell_timer_label.time = time_left
        end
        timers[index] = cell_timer_label
    end

    local button_created_label = cell:find_child('cell_button_created_caption', true, false)
    local created_caption = cross_promo.button_accept_caption
    if button_created_label and button_created_caption ~= "" then
        button_created_label.caption = created_caption
    end

    local button_accepted_caption = cell:find_child('cell_button_accepted_caption', true, false)
    local accepted_caption = cross_promo.button_go_link_caption
    if button_accepted_caption and accepted_caption ~= "" then
        button_accepted_caption.caption = accepted_caption
    end

    update_cell(cell, offer)

    return cell
end

function offers_table_delegate:get_cell_count()
    return #offers
end
