/* fragment water shader for earth */

#ifdef GL_ES
precision mediump float;
#define LOWP lowp
#define MEDIUMP mediump
#else
#define LOWP
#define MEDIUMP
#endif

varying vec2 tCoords;
varying vec2 tCoordsBrush1;
varying vec2 tCoordsBrush2;
varying LOWP vec4 color;
uniform LOWP sampler2D texture;
uniform LOWP sampler2D texture_brush1;
uniform LOWP sampler2D texture_brush2;


const LOWP vec3  kRGBToI     = vec3 (0.596, -0.275, -0.321);
const LOWP vec3  kRGBToQ     = vec3 (0.212, -0.523, 0.311);

MEDIUMP vec2 hue_from_rgb(LOWP vec3 color) {
    MEDIUMP float   I      = dot (color, kRGBToI);
    MEDIUMP float   Q      = dot (color, kRGBToQ);
    
    return vec2(Q,I);
}

const LOWP vec2 hue_reference = vec2(-0.967843175, 0.251553953);   // reference hue value (blue)
const MEDIUMP float hue_scale = 1208.0/128.0;//
const MEDIUMP float hue_offset = 19.0/128.0;//

void main() {
    LOWP vec4 tex = texture2D(texture, tCoords);
    MEDIUMP vec2 hue = hue_from_rgb(tex.rgb)*hue_scale;
    LOWP float a = tex.a;
    LOWP float d1 = hue_offset - dot(hue,hue_reference);
    //if (d1<=0.0) discard;
    d1 = max(0.0,d1);
    //float d1 = min(abs(hue-hue_reference),abs(hue+6.28-hue_reference))*1.5;  /// distance in hue space for ocean only
    //d1 = clamp(hue_offset - d1,0.0,1.0);
    LOWP vec3 tbrush = texture2D(texture_brush1,tCoordsBrush1).rgb;          /// use one texture as two tiles
    LOWP vec3 tbrush2 = texture2D(texture_brush2,tCoordsBrush2).rgb;
    //tbrush.a = 0.0;
    //tbrush2.a = 0.0;
    //gl_FragColor = vec4(d1*a,0,0,a);  // for checking color detection algorihm
    gl_FragColor = color*vec4(tex.rgb + (tbrush*tbrush2)*d1*a*a , a );
}