local barn_animator=nil
local close_animation_in_process = false
local max_visible_cell = 3

function get_texture_name(texture)
	local texture_by_device = texture
	if g_device.is_tablet then
		texture_by_device = texture_by_device .. "_ipad"
		print(texture_by_device)
	end
	return texture_by_device
end


function on_init_dialog()

	dialog.theme_flags = 2
    dialog.bevel_type = milk_bevel_type_none

	local reward_vector = framework:game():get_payment_reward_item_vector()

	if #reward_vector == 0 then
		dialog:end_dialog()
	end

	if #reward_vector == 1 then
		dialog:load_content('in_app_purchaser_one_reward_success_msgbox')
	else 
		if #reward_vector == 2 then
			dialog:load_content('in_app_purchaser_two_reward_success_msgbox')
		else
			if #reward_vector > 2 then
				dialog:load_content('in_app_purchaser_three_reward_success_msgbox')
			end
		end
	end		

	-- congradulation title
	local congradulation_title = dialog:find_child("congradulation_title", true, false)    
	if congradulation_title then		
		congradulation_title.caption = framework:strings():load("*InAppPurchaserGiftTitle", "Gift")
	end

	-- description title
	local bought_mb = money_barter_item.amount + money_barter_item.bonus_display_amount
	local description_str = framework:strings():load("*CityYouGetMegabucksSuccessfullyWithReward", "You have successfully obtained %d megabucks and next gifts:")
	
	description_str = string.format(description_str, bought_mb, reward_item_string);

	local description_title = dialog:find_child("description_title", true, false)    
	if description_title then
		description_title.caption = description_str
	end

	
	local spinboard = dialog:find_child("spinboard",true, false)
	if spinboard then
	
		if #framework:game():get_payment_reward_item_vector() <= max_visible_cell then
			spinboard.enable_horizontal_scroll = false
		end 
	
		spinboard:update_content()
	end

end

function init_barn_animator()
    if not barn_animator then
        local name = "city_barn_animation_impl"
        barn_animator = cclasses[name]
        barn_animator=barn_animator.new()
    end
end

function on_post_init_dialog()
	init_barn_animator()
end

spinboard_delegate = {
	create_cell = function (  self, idx, rect )
		local w = self.spinboard:create_child(rect)
		
		local loc_idx = idx+1
                
        framework:json_ui_manager():build_from_resource("in_app_purchaser_cell_success_reward",w)
        w.rect = rect
        
        local reward_item = framework:game():get_payment_reward_item_vector()[loc_idx]
        action_object_info = framework:object_factory():get_object_info_by_type_name(reward_item.name)        
           
        
		if action_object_info then       
			local reward_name_label = w:find_child('reward_name',true,false)
			if reward_name_label then
				reward_name_label.caption = action_object_info.description
			end

			local cell_bg = w:find_child('cell_bg', true, false)
			local info_plate = w:find_child("info_plate", true, false)
			
			if		action_object_info.shop_department == "bouquets" 
				or	action_object_info.shop_department == "act_of_expanding"
				or	action_object_info.shop_department == "expand"
				or	action_object_info.shop_department == "materials"
				or	action_object_info.shop_department == "craft_materials"
			then
			
				if info_plate then
					info_plate.visible = false
				end
			
				if cell_bg then
					local reward_image = cell_bg:find_child('reward_image', true, false)
					if reward_image then
						reward_image.texture_name = action_object_info.shop_image_path
						reward_image:set_ready_progress_as_indicator()
						reward_image.tag = "award_widget_" .. idx
					end
	
					local reward_quantity_label = cell_bg:find_child('reward_quantity', true, false)
					if reward_quantity_label then
						reward_quantity_label.caption = "x" .. tostring(reward_item.quantity)
					end
				end
			
			else
				
				if cell_bg then
					cell_bg.visible = false
				end
			
				if info_plate then
					info_plate.visible = true
					info_plate.object_name = action_object_info.type_name
					info_plate:layout_controls()
					info_plate:add_show_property(21, reward_item.quantity, true)
					
					local reward_image = info_plate:find_child('object_image', true, false)
					if reward_image then
						reward_image.tag = "award_widget_" .. idx
					end 
					
					local background_image = info_plate:find_child('default_background_image', true, false)
					if background_image then
						background_image.image_name = get_texture_name("advances_small_item")
					end
					
				end
					
			end				
		end

		w.tag = "cell_" .. idx

		return w
	end,
	
	get_cell_count = function ( )
		return #framework:game():get_payment_reward_item_vector()
    end
}

function on_ok_btn_action()
	if show_barn_update_animation() == false then
		dialog:end_dialog()
	end
end

function on_close_button_action()
	if show_barn_update_animation() == fale then
		dialog:end_dialog()
	end
end

function show_barn_update_animation()

	if close_animation_in_process == true then
		return true
	end

	reward_rects = {}
	type_names = {}

	for i= 1,#framework:game():get_payment_reward_item_vector() do 
	
		idx = i - 1
	
		local reward_image = dialog:find_child('award_widget_' .. idx, true, false)
		if reward_image then
			local reward_item = framework:game():get_payment_reward_item_vector()[i]
			type_names[i] = reward_item.name			
		end
	end

	close_window = true
	
	if #type_names > 0 and barn_animator then
		close_animation_in_process = true
        barn_animator:show_barn_update_animation_spinboard(type_names, close_window)
    else
		return false
    end
    
    return true
end

function on_end_dialog()
	if show_barn_update_animation() == true then
		return false
	else
		return true
	end
end

function end_dialog()
	framework:game():clear_payment_reward_item_vector()
end
