
local remaining_time = 0
local num_tick = 0
local increase_interval = 86400
local tick_interval = 1000
local image_suffix = "_progress_kakao"

local quests_list = {}

function on_init_dialog()

	if event then
    	dialog.caption = event.quest_event_base.name
    end

    update_event_quests_list()

	dialog:load_content("kakao_event_dialog")

	local spinboard = dialog:find_child("spinboard", true, false)
	if spinboard then
        spinboard:update_content()
	end

    local person = dialog:find_child("person", true, false)
    if person then
        person.texture_name = get_texture_name(event.quest_event_base.person)
    end

    local object_info = dialog:find_child("object_info", true, false)
    if object_info then
        local properties = 24;
        object_info.object_name = reward_object.type_name

        if reward_object.power_consume == 0 and not(reward_object.power_produce == 0) then
            properties = properties + 1
        end
        if reward_object.water_consume == 0 and not(reward_object.water_produce == 0) then
            properties = properties + 2
        end
        object_info.properties = properties
        
        object_info:layout_controls()
    end

    local object_name = dialog:find_child("object_name", true, false)
    if object_name then
        object_name.caption = reward_object.description
    end
   
    local ext_button_label = dialog:find_child("ext_button_text", true, false)
    if ext_button_label then
        if not event.quest_event_base.increase_time then
            ext_button_label.visible = false
        end
        ext_button_label.caption = string.format(ext_button_label.caption, event.quest_event_base.increase_time_cost)
    end

    local description = dialog:find_child("description", true, false)
    if description then
        description.caption = event.quest_event_base.description
    end

    update_remaining_time()
    update_ext_buttom()

end

function update_event_quests_list()
    for i = 1, #event:get_quests_list() do
        if not(event:get_quests_list()[i].id == event.quest_event_base.id) then
            table.insert(quests_list, event:get_quests_list()[i])
        end
    end
    table.sort(quests_list, function(a, b) return a.id < b.id end)
end

function get_texture_name(texture)
    texture = texture .. image_suffix
    if g_device.is_tablet then
        texture = texture .. "_ipad"
    end
    if g_device.is_hdpi then
        texture = texture .. "@2x"
    end
    return texture
end

function on_reward_action()
    print("[kakao_event_dialog] reward action")
    
end

function update_remaining_time()
    remaining_time = event.quest_event_base.end_date_time - framework:get_server_time_sec()
    if remaining_time > 0 then
        return remaining_time
    end
    return 0
end

function is_enough_money()
    return framework:game().gold > event.quest_event_base.increase_time_cost
end

function on_ext_button_action()
    print("[kakao_event_dialog] increase event time action")
    if not is_enough_money() then
        framework:dialogs():show_not_enough_bucks_dialog(3)
    else
        if framework:settings_controller().gold_warning_enabled then
            params = {}
            params.product = framework:strings():load("*CityBuyEventTimeForMegabucks", "extra time")
            params.price = event.quest_event_base.increase_time_cost
            params.ok_callback = increase_event_time_confirm
            params.no_callback = increase_event_time_cancel
            framework:dialogs():show_buy_for_megabucks_dialog(params)
        else
            increase_event_time_confirm()
        end
    end
end

function increase_event_time_confirm()
    local command_class = cclasses["sqc_quest_increase_event_time"]
    if command_class then
        local command = command_class.new(event.quest_event_base.id, framework:game().room_id)
        command:append_complete_callback(increase_event_time_complete)
        command:append_error_callback(increase_event_time_error)
        framework:cmd_processor():schedule(command)
    else
        error("[kakao_event_dialog] cant't found command class increase_event_time")
    end 
end

function increase_event_time_complete()
    print("[kakao_event_dialog] command increase_event_time complete success")
    framework.quests_processor:increase_event_time(event.quest_event_base.id)
    update_remaining_time()
    update_ext_buttom()
end

function increase_event_time_error()
    print("[kakao_event_dialog] command increase_event_time complete error")
end

function increase_event_time_cancel()
    print("[kakao_event_dialog] increase event time canceld")
end

function on_ext_button_unactive_action()
   print("[kakao_event_dialog] increase event time unactive action")
   local params = {}
   -- 0: sqc_message_box_type_default
   -- 1: sqc_message_box_type_local_error
   -- 2: sqc_message_box_type_server_error
   params.type = 1
   params.dlg_title = framework:strings():load("*CityBuyError", "Error");
   params.dlg_text = framework:strings():load("*CityTimeLimitNotCome", "Extend the time only 24 hours before the end of the event");
   params.button_text = framework:strings():load("*CityOKButton", "OK" );
   framework:dialogs():show_message_box(params);
end

function update_ext_buttom()
    if event.quest_event_base.increase_time then
        activate_ext_button(remaining_time < increase_interval)
    end
end

function activate_ext_button(activate)
    local ext_button = dialog:find_child("ext_button", true, false)
    if ext_button then
        ext_button.active = activate
        if activate then
            ext_button.button_type = 8 --> red
        else
            ext_button.button_type = 9 --> gift_gray
        end
    end
end

function update_timer()
    local timer = dialog:find_child("remaining_time", true, false)
    if timer then
        timer.time = remaining_time
    end
end

function on_timer(tick)
    if dialog then
        if tick > num_tick then
            if remaining_time > 0 then
                if tick - num_tick >= 2 then
                    update_remaining_time()
                end
                remaining_time = remaining_time - 1
                update_timer()
                update_ext_buttom()
            else 
                if not event.is_active then
                    dialog:end_dialog()
                end
            end
            num_tick = num_tick + tick_interval
        end
    end
end

-- spinboard delegate

spinboard_delegate = {}

function spinboard_delegate:create_cell(idx, rect)  
    if not #quests_list then
        return nil
    end

    local index = idx + 1

    local cell = self.spinboard:create_child(rect)
    framework:json_ui_manager():build_from_resource("kakao_event_cell", cell)
    
    local quest = quests_list[index]
    local cell_icon = cell:find_child("cell_icon", true, false)
    if cell_icon then
        cell_icon.texture_name = get_texture_name(quest.image)
    end

    local cell_title = cell:find_child("cell_title", true, false)
    if cell_title then
        cell_title.caption = quest.name
    end

    local cell_reward_count = cell:find_child("cell_reward_count", true, false)
    if cell_reward_count then
        cell_reward_count.caption = tostring(quest.total_count)
    end

    local cell_action = function() framework:dialogs():show_event_goals(quest.id) 
                                   dialog:end_dialog() end

    local cell_button = cell:find_child("cell_button", true, false)
    if cell_button then
        if quest.check_reuieres then
            cell_button.action = cell_action
        end
        if (quest.check_reuieres) and not(quest.completed) then
            cell_button.visible = true
        end
    end

    local cell_bg = cell:find_child("cell_bg", true, false)
    if cell_bg then
        if quest.check_reuieres then
            cell_bg.action = cell_action
        end
        if quest.completed then
            cell_bg.action = nil
            background = "kakao_event_item_green_bg"
            if g_device.is_tablet then
                background = background .. "_ipad" 
            end
            cell_bg.image_name = background
            cell_bg.pressed_image = background
            cell_bg.inactive_image = background
        end
    end

    local cell_completed_icon = cell:find_child("cell_completed_icon", true, false)
    if cell_completed_icon then
        if quest.completed then
            cell_completed_icon.visible = true
        end
    end
        
    local cell_separator = cell:find_child("separator")
    if cell_separator then
        if index == #quests_list then
            cell_separator.visible = false
        end
    end

    local cell_separator_finish = cell:find_child("separator_finish", true, false)
    if cell_separator_finish then
        if quest.completed then
            cell_separator.visible = false;
            cell_separator_finish.visible = true;
        end
    end

    return cell
end

function spinboard_delegate:get_cell_count()
    return #quests_list
end

