function create_research_bubble(research_item, controller, parent)
	local handler = { bubble, start_callback, force_callback }

	function handler:force_research(research_item)
		local force_dlg = cclasses.sqc_scriptable_dialog.create(framework:windows().desktop, "force_object_msgbox.lua")
	
		local object_info = framework:object_factory():get_object_info_by_type_name(research_item.contract_name)

		if force_dlg and object_info then
			local level_text = framework:strings():load("*TransportResearchTreeLevelCell", "lev.")
			local research_force_price = controller:research_force_price(research_item)	

			force_dlg.scripting_object.m_object_info = object_info	
			force_dlg.scripting_object.object_name = research_item.name .. " " .. level_text .. object_info.next_contract_level .."/".. research_item.need_level
			force_dlg.scripting_object.force_cost = research_force_price
			force_dlg.scripting_object.force_callback = function()
															self:force_callback()
														end
			force_dlg:do_modal()
		end
	end

	

	function handler:init()
		self.bubble = framework:json_ui_manager():create_from_resource("research_bubble", parent)

		local big_bubble = self.bubble:find_child("bubble_big_bg", true, false)
		local small_bubble = self.bubble:find_child("bubble_small_bg", true, false)
		if big_bubble and small_bubble then
			if controller:research_done(research_item) == true then
				big_bubble.visible = false
				small_bubble.visible = true

				local research_info = small_bubble:find_child("research_info", true, false)
				local complete_research = small_bubble:find_child("complete_research", true, false)
			
				complete_research.visible = true
				research_info.visible = false

				local research_name = complete_research:find_child("research_name", true, false)
				research_name.caption = research_item.name
			
			elseif research_item.check_requires == true and (controller:has_progress_research(controller.transport_type) == false or research_item.research_in_progress == true) then
				big_bubble.visible = true
				small_bubble.visible = false

				local research_name = big_bubble:find_child("research_name", true, false)
				if research_name then
					research_name.caption = research_item.name
				end
				
				local research_button = big_bubble:find_child("research_button", true, false)
				local force_button = big_bubble:find_child("force_button", true, false)
				local timer = big_bubble:find_child("timer_bg", true, false)
				if research_item.research_in_progress == true then
					research_button.visible = false
					force_button.visible = true
					timer.visible = true

					force_button.action = function()
											self:on_destroy()
											self:force_research(research_item)
										end

					scripting_server:start_thread(function() 
													while true do
														local timer = self.bubble:find_child("timer_label", true, false)
														if timer then
															timer.time = controller:research_in_progress_time(research_item) / 1000
														end
														coroutine.yield(1) 
													end
					 							end)
				else
					research_button.visible = true
					force_button.visible = false
					timer.visible = false

					research_button.action = function()
											self:on_destroy()
											controller:start_research(research_item)
										end
				end
			else
				big_bubble.visible = false
				small_bubble.visible = true

				local research_info = small_bubble:find_child("research_info", true, false)
				local complete_research = small_bubble:find_child("complete_research", true, false)
			
				complete_research.visible = false
				research_info.visible = true

				local research_name = research_info:find_child("research_name", true, false)
				research_name.caption = research_item.name
			end
		end
	end

	function handler:set_rect(rect)
		self.bubble.rect = rect
	end

	function handler:get_rect()
		return self.bubble.rect
	end

	function handler:on_destroy()
		self.bubble:destroy()
	end

	return handler

end