load_script("research_bubble.lua")

local progress_widget = {}

local train_transport_design = {}

local plate_positions = {
    [0] = "top",
    [1] = "center",
    [2] = "bottom",
}

local plate_types = {
    [0] = "design_cell_scheme",
    [1] = "design_cell_research_top_bottom"
}

local arrow_types = {
    [0] = "arrow_h_t_left",
    [1] = "arrow_h_t_right",
    [2] = "arrow_h_t_long",
    [3] = "arrow_h_b_left",
    [4] = "arrow_h_b_right",
    [5] = "arrow_h_b_long",
    [6] = "arrow_h_c_left",
    [7] = "arrow_h_c_right",
    [8] = "arrow_h_c_long",
    [9] = "arrow_v_t_b",
}

local arrow_color_names = {
    ["blue"] = {
                    ["arrow_h_t_left"]  = "arrow_min_blue_1",
                    ["arrow_h_t_right"] = "arrow_min_blue_2",
                    ["arrow_h_t_long"]  = "arrow_min_blue_1",
                    ["arrow_h_b_left"]  = "arrow_min_blue_1",
                    ["arrow_h_b_right"] = "arrow_min_blue_2",
                    ["arrow_h_b_long"]  = "arrow_min_blue_1",
                    ["arrow_h_c_left"]  = "arrow_min_blue_1",
                    ["arrow_h_c_right"] = "arrow_min_blue_2",
                    ["arrow_h_c_long"]  = "arrow_min_blue_1",
                    ["arrow_v_t_b"]  = "arrow_min_ver_blue",
                }, 
    ["yellow"] = {
                    ["arrow_h_t_left"]  = "arrow_min_yellow_1",
                    ["arrow_h_t_right"] = "arrow_min_yellow_2",
                    ["arrow_h_t_long"]  = "arrow_min_yellow_1",
                    ["arrow_h_b_left"]  = "arrow_min_yellow_1",
                    ["arrow_h_b_right"] = "arrow_min_yellow_2",
                    ["arrow_h_b_long"]  = "arrow_min_yellow_1",
                    ["arrow_h_c_left"]  = "arrow_min_yellow_1",
                    ["arrow_h_c_right"] = "arrow_min_yellow_2",
                    ["arrow_h_c_long"]  = "arrow_min_yellow_1",
                    ["arrow_v_t_b"]  = "arrow_min_ver_yellow",
                },  
    ["grey"] = {
                    ["arrow_h_t_left"]  = "arrow_min_grey_1",
                    ["arrow_h_t_right"] = "arrow_min_grey_2",
                    ["arrow_h_t_long"]  = "arrow_min_grey_1",
                    ["arrow_h_b_left"]  = "arrow_min_grey_1",
                    ["arrow_h_b_right"] = "arrow_min_grey_2",
                    ["arrow_h_b_long"]  = "arrow_min_grey_1",
                    ["arrow_h_c_left"]  = "arrow_min_grey_1",
                    ["arrow_h_c_right"] = "arrow_min_grey_2",
                    ["arrow_h_c_long"]  = "arrow_min_grey_1",
                    ["arrow_v_t_b"]  = "arrow_min_ver_grey",
                },
}

local plate_color_names = {
    ["blue"] = {
                    ["design_cell_research_top_bottom"]  = "item_min_blue",
                    ["design_cell_scheme"] = "item_big_blue",
                },
    ["yellow"] = {
                    ["design_cell_research_top_bottom"]  = "item_min_yellow",
                    ["design_cell_scheme"] = "item_big_yellow",
                },
    ["grey"] = {
                    ["design_cell_research_top_bottom"]  = "item_min_grey",
                    ["design_cell_scheme"] = "item_big_grey",
                },
    ["green"] = {
                    ["design_cell_research_top_bottom"]  = "item_min_green",
                    ["design_cell_scheme"] = "item_big_green",
                },
}

local cell_info_local_1 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            contract = "train_transport0",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            --ref = "mountains_unlock",
            unlock_to_hide = "drawing_train_transport1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            --ref = "mountains_unlock",
            unlock_to_hide = "drawing_train_transport1"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "blue",
            --ref = "mountains_unlock",
            unlock_to_show = "drawing_train_transport1"
        }
    }
}

local cell_info_local_2 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_chassis_unlock1",
            contract = "train_chassis"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_chassis_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport1"
}

local cell_info_local_3 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_electric_motor_unlock1",
            contract = "train_electric_motor"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_electric_motor_unlock1"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_chassis_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport1"
}

local cell_info_local_4 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_body_unlock1",
            contract = "train_body"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_chassis_unlock2",
            contract = "train_chassis"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_body_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_chassis_unlock2"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            --ref = "mountains_unlock"
        },
        [4] = {
            type = "arrow_h_c_long",
            color = "yellow",
            ref = "train_electric_motor_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport1"
}

local cell_info_local_5 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            ref = "drawing_train_transport1",
            contract = "train_transport1",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "drawing_train_transport1",
            unlock_to_hide = "drawing_train_transport2"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "drawing_train_transport1",
            unlock_to_hide = "drawing_train_transport2"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_body_unlock1",
            unlock_to_hide = "drawing_train_transport1"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_chassis_unlock2",
            unlock_to_hide = "drawing_train_transport1"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_electric_motor_unlock1",
            unlock_to_hide = "drawing_train_transport1"
        },
        [5] = {
            type = "arrow_h_c_right",
            color = "blue",
            unlock_to_show = "drawing_train_transport1"
        },
        [6] = {
            type = "arrow_h_c_left",
            color = "blue",
            unlock_to_show = "drawing_train_transport2"
        }
    }
}

local cell_info_local_6 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_equipment_unlock1",
            contract = "train_equipment"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_wheelsets_unlock1",
            contract = "train_wheelsets"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_equipment_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_wheelsets_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "drawing_train_transport1"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "drawing_train_transport1"
        }
    },
    unlock_to_hide = "drawing_train_transport2"
}

local cell_info_local_7 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_braking_system_unlock1",
            contract = "train_braking_system"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_pneumatics_unlock1",
            contract = "train_pneumatics"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_braking_system_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_pneumatics_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_equipment_unlock1"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_wheelsets_unlock1"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "blue",
            ref = "train_braking_system_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport2"
}

local cell_info_local_8 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            ref = "drawing_train_transport2",
            contract = "train_transport2",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "drawing_train_transport2",
            unlock_to_hide = "drawing_train_transport3"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "drawing_train_transport2",
            unlock_to_hide = "drawing_train_transport3"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_braking_system_unlock1",
            unlock_to_hide = "drawing_train_transport2"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_pneumatics_unlock1",
            unlock_to_hide = "drawing_train_transport2"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "blue",
            unlock_to_show = "drawing_train_transport2"
        },
        [5] = {
            type = "arrow_h_c_left",
            color = "blue",
            unlock_to_show = "drawing_train_transport3"
        }
    }
}

local cell_info_local_9 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_chassis_unlock3",
            contract = "train_chassis"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_batteries_unlock1",
            contract = "train_batteries"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_chassis_unlock3"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_batteries_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "drawing_train_transport2"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "drawing_train_transport2"
        }
    },
    unlock_to_hide = "drawing_train_transport3"
}

local cell_info_local_10 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_body_frame_unlock1",
            contract = "train_body_frame"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_undercar_generator_unlock1",
            contract = "train_undercar_generator"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_body_frame_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_undercar_generator_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_chassis_unlock3"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_batteries_unlock1"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "blue",
            ref = "train_body_frame_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport3"
}

local cell_info_local_11 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_streamlined_body_unlock2",
            contract = "train_streamlined_body"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_body_frame_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_undercar_generator_unlock1"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_streamlined_body_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport3"
}

local cell_info_local_12 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_electric_motor_unlock2",
            contract = "train_electric_motor"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_body_frame_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_undercar_generator_unlock1"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_electric_motor_unlock2"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_streamlined_body_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport3"
}

--- level: {min_quantity: -999999, unlocker: drawing_train_transport2}
--        - level: {min_quantity: -999999, unlocker: train_chassis_unlock3}
--        - level: {min_quantity: -999999, unlocker: train_body_frame_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_batteries_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_undercar_generator_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_electric_motor_unlock2}

local cell_info_local_13 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            ref = "drawing_train_transport3",
            contract = "train_transport3",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "drawing_train_transport3",
            unlock_to_hide = "drawing_train_transport4"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "drawing_train_transport3",
            unlock_to_hide = "drawing_train_transport4"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_body_frame_unlock1",
            unlock_to_hide = "drawing_train_transport3"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_undercar_generator_unlock1",
            unlock_to_hide = "drawing_train_transport3"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_electric_motor_unlock2",
            unlock_to_hide = "drawing_train_transport3"
        },
        [5] = {
            type = "arrow_h_c_right",
            color = "blue",
            unlock_to_show = "drawing_train_transport3"
        },
        [6] = {
            type = "arrow_h_c_left",
            color = "blue",
            unlock_to_show = "drawing_train_transport4"
        }
    }
}

--        - level: {min_quantity: -999999, unlocker: drawing_train_transport3}
--        - level: {min_quantity: -999999, unlocker: train_body_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_lighting_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_brake_cylinders_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_main_transformer_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_chassis_unlock5}
--        - level: {min_quantity: -999999, unlocker: train_damper_unlock1}
local cell_info_local_14 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_body_unlock2",
            contract = "train_body"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_main_transformer_unlock1",
            contract = "train_main_transformer"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_body_unlock2"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_main_transformer_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "drawing_train_transport3"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "drawing_train_transport3"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "blue",
            ref = "train_body_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport4"
}

local cell_info_local_15 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_lighting_unlock1",
            contract = "train_lighting"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_chassis_unlock5",
            contract = "train_chassis"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_lighting_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_chassis_unlock5"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_body_unlock2"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_main_transformer_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport4"
}

local cell_info_local_16 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_brake_cylinders_unlock1",
            contract = "train_brake_cylinders"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_brake_cylinders_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_chassis_unlock5"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_lighting_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport4"
}

local cell_info_local_17 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_damper_unlock1",
            contract = "train_damper"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_brake_cylinders_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_chassis_unlock5"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_damper_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport4"
}

local cell_info_local_18 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            ref = "drawing_train_transport4",
            contract = "train_transport4",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "drawing_train_transport4",
            unlock_to_hide = "drawing_train_transport5"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "drawing_train_transport4",
            unlock_to_hide = "drawing_train_transport5"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_brake_cylinders_unlock1",
            unlock_to_hide = "drawing_train_transport4"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_chassis_unlock5",
            unlock_to_hide = "drawing_train_transport4"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_damper_unlock1",
            unlock_to_hide = "drawing_train_transport4"
        },
        [5] = {
            type = "arrow_h_c_right",
            color = "blue",
            unlock_to_show = "drawing_train_transport4"
        },
        [6] = {
            type = "arrow_h_c_left",
            color = "blue",
            unlock_to_show = "drawing_train_transport5"
        }
    }
}

--- level: {min_quantity: -999999, unlocker: drawing_train_transport4}
--        - level: {min_quantity: -999999, unlocker: train_electric_motor_unlock3}
--        - level: {min_quantity: -999999, unlocker: train_braking_resistor_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_brake_control_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_autonomous_generator_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_emergency_power_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_braking_system_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_impact_absorber_unlock1}
        
local cell_info_local_19 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_electric_motor_unlock3",
            contract = "train_electric_motor"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_autonomous_generator_unlock1",
            contract = "train_autonomous_generator"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_electric_motor_unlock3"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_autonomous_generator_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "drawing_train_transport4"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "drawing_train_transport4"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "blue",
            ref = "train_electric_motor_unlock3"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_20 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_braking_resistor_unlock1",
            contract = "train_braking_resistor"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_emergency_power_unlock1",
            contract = "train_emergency_power"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_braking_resistor_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_emergency_power_unlock1"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_electric_motor_unlock3"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_autonomous_generator_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_21 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "blue",
            ref = "train_brake_control_unlock1",
            contract = "train_brake_control"
        },
        ["bottom"] = {
            color = "blue",
            ref = "train_braking_system_unlock2",
            contract = "train_braking_system"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_left",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_braking_resistor_unlock1"
        },
        [3] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_emergency_power_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_22 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_wheelsets_unlock2",
            contract = "train_wheelsets"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_wheelsets_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_23 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_streamlined_body_unlock3",
            contract = "train_streamlined_body"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_streamlined_body_unlock3"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_wheelsets_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_24 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_chassis_unlock6",
            contract = "train_chassis"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_chassis_unlock6"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_streamlined_body_unlock3"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_25 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "train_impact_absorber_unlock1",
            contract = "train_impact_absorber"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "train_impact_absorber_unlock1"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_chassis_unlock6"
        }
    },
    unlock_to_hide = "drawing_train_transport5"
}

local cell_info_local_26 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "blue",
            ref = "drawing_train_transport5",
            contract = "train_transport5",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "blue",
            ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "blue",
            ref = "train_braking_system_unlock2"
        },
        [2] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "train_impact_absorber_unlock1"
        },
        [3] = {
            type = "arrow_h_c_right",
            color = "blue",
            unlock_to_show = "drawing_train_transport5"
        }
    }
}

local cells_info_local = {
    [0] = cell_info_local_1,
    [1] = cell_info_local_2,
    [2] = cell_info_local_3,
    [3] = cell_info_local_4,
    [4] = cell_info_local_5,
    [5] = cell_info_local_6,
    [6] = cell_info_local_7,
    [7] = cell_info_local_8,
    [8] = cell_info_local_9,
    [9] = cell_info_local_10,
    [10] = cell_info_local_11,
    [11] = cell_info_local_12,
    [12] = cell_info_local_13,
    [13] = cell_info_local_14,
    [14] = cell_info_local_15,
    [15] = cell_info_local_16,
    [16] = cell_info_local_17,
    [17] = cell_info_local_18,
    [18] = cell_info_local_19,
    [19] = cell_info_local_20,
    [20] = cell_info_local_21,
    [21] = cell_info_local_22,
    [22] = cell_info_local_23,
    [23] = cell_info_local_24,
    [24] = cell_info_local_25,
    [25] = cell_info_local_26,
}

local cell_info_world_1 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "yellow",
            contract = "train_transport7",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_left",
            color = "yellow",
            --ref = "train_transport7",
            unlock_to_hide = "drawing_train_transport8"
        },
        [1] = {
            type = "arrow_h_c_left",
            color = "yellow",
            --ref = "train_transport7",
            unlock_to_show = "drawing_train_transport8"
        }
    }
}

---        - level: {min_quantity: -999999, unlocker: train_chassis_unlock1}
---        - level: {min_quantity: -999999, unlocker: train_electric_motor_unlock1}
---        - level: {min_quantity: -999999, unlocker: train_streamlined_body_unlock1}
---        - level: {min_quantity: -999999, unlocker: train_air_suspension_unlock1}
---        - drawing_train_transport8: {min_quantity: 1} 

local cell_info_world_2 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_chassis_unlock1",
            contract = "train_chassis"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ---ref = "train_brake_control_unlock1"
        },
        [1] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_chassis_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport8"
}

local cell_info_world_3 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_electric_motor_unlock1",
            contract = "train_electric_motor"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_streamlined_body_unlock1",
            contract = "train_streamlined_body"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_chassis_unlock1"
        },
        [1] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_electric_motor_unlock1"
        },
        [2] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_streamlined_body_unlock1"
        },
        [3] = {
            type = "arrow_v_t_b",
            color = "yellow",
            ref = "train_electric_motor_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport8"
}

local cell_info_world_4 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["bottom"] = {
            color = "yellow",
            ref = "train_air_suspension_unlock1",
            contract = "train_air_suspension"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "yellow",
            ref = "train_electric_motor_unlock1"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_streamlined_body_unlock1"
        },
        [2] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_air_suspension_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport8"
}

local cell_info_world_5 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "drawing_train_transport8",
            contract = "train_transport8",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_electric_motor_unlock1",
            unlock_to_hide = "drawing_train_transport8"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_air_suspension_unlock1",
            unlock_to_hide = "drawing_train_transport8"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "drawing_train_transport8",
            unlock_to_hide = "drawing_train_transport9"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "drawing_train_transport8",
            unlock_to_hide = "drawing_train_transport9"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "yellow",
            --ref = "drawing_train_transport7",
            unlock_to_show = "drawing_train_transport8"
        },
        [5] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "drawing_train_transport8",
            unlock_to_show = "drawing_train_transport9"
        },
    }
}

--        - level: {min_quantity: -999999, unlocker: drawing_train_transport8}
--        - level: {min_quantity: -999999, unlocker: train_streamlined_body_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_electric_motor_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_air_suspension_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_chassis_unlock4}

local cell_info_world_6 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_streamlined_body_unlock2",
            contract = "train_streamlined_body"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_air_suspension_unlock2",
            contract = "train_air_suspension"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "drawing_train_transport8"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "drawing_train_transport8"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_streamlined_body_unlock2"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_air_suspension_unlock2"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "yellow",
            ref = "train_streamlined_body_unlock2"
        }
    },
    unlock_to_hide = "drawing_train_transport9"
}

local cell_info_world_7 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_electric_motor_unlock2",
            contract = "train_electric_motor"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_chassis_unlock4",
            contract = "train_chassis"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_streamlined_body_unlock2"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_air_suspension_unlock2"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_electric_motor_unlock2"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_chassis_unlock4"
        }
    },
    unlock_to_hide = "drawing_train_transport9"
}

local cell_info_world_8 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "drawing_train_transport9",
            contract = "train_transport9",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_electric_motor_unlock2",
            unlock_to_hide = "drawing_train_transport9"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_chassis_unlock4",
            unlock_to_hide = "drawing_train_transport9"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "drawing_train_transport9",
            unlock_to_hide = "drawing_train_transport10"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "drawing_train_transport9",
            unlock_to_hide = "drawing_train_transport10"
        },
        [4] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "drawing_train_transport8",
            unlock_to_show = "drawing_train_transport9"
        },
        [5] = {
            type = "arrow_h_c_left",
            color = "yellow",
            ref = "drawing_train_transport9",
            unlock_to_show = "drawing_train_transport10"
        },
    }
}

--- level: {min_quantity: -999999, unlocker: drawing_train_transport9}
--        - level: {min_quantity: -999999, unlocker: train_damper_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_sleeps_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_conditioners_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_radio_warning_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_wheelsets_unlock2}
--        - level: {min_quantity: -999999, unlocker: train_streamlined_body_unlock3}
--        - level: {min_quantity: -999999, unlocker: train_chassis_unlock6}
--        - level: {min_quantity: -999999, unlocker: train_impact_absorber_unlock1}
--        - level: {min_quantity: -999999, unlocker: train_air_suspension_unlock3}
--        - level: {min_quantity: -999999, unlocker: train_automatic_couplers_unlock1}

local cell_info_world_9 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_damper_unlock1",
            contract = "train_damper"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_conditioners_unlock1",
            contract = "train_conditioners"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "drawing_train_transport9"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "drawing_train_transport9"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_damper_unlock1"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_conditioners_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_10 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_wheelsets_unlock2",
            contract = "train_wheelsets"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_radio_warning_unlock1",
            contract = "train_radio_warning"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_damper_unlock1"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_conditioners_unlock1"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_wheelsets_unlock2"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_radio_warning_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_11 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["bottom"] = {
            color = "yellow",
            ref = "train_sleeps_unlock1",
            contract = "train_sleeps"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_long",
            color = "yellow",
            ref = "train_wheelsets_unlock2"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_radio_warning_unlock1"
        },
        [2] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_sleeps_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_12 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_streamlined_body_unlock3",
            contract = "train_streamlined_body"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_air_suspension_unlock3",
            contract = "train_air_suspension"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_wheelsets_unlock2"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_sleeps_unlock1"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_streamlined_body_unlock3"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_air_suspension_unlock3"
        },
        [4] = {
            type = "arrow_v_t_b",
            color = "yellow",
            ref = "train_streamlined_body_unlock3"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_13 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_chassis_unlock6",
            contract = "train_chassis"
        },
        ["bottom"] = {
            color = "yellow",
            ref = "train_automatic_couplers_unlock1",
            contract = "train_automatic_couplers"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_streamlined_body_unlock3"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_air_suspension_unlock3"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_chassis_unlock6"
        },
        [3] = {
            type = "arrow_h_b_left",
            color = "yellow",
            ref = "train_automatic_couplers_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_14 = {
    type = "design_cell_research_top_bottom",
    items = {
        ["top"] = {
            color = "yellow",
            ref = "train_impact_absorber_unlock1",
            contract = "train_impact_absorber"
        }
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_chassis_unlock6"
        },
        [1] = {
            type = "arrow_h_b_long",
            color = "yellow",
            ref = "train_automatic_couplers_unlock1"
        },
        [2] = {
            type = "arrow_h_t_left",
            color = "yellow",
            ref = "train_impact_absorber_unlock1"
        }
    },
    unlock_to_hide = "drawing_train_transport10"
}

local cell_info_world_15 = {
    type = "design_cell_scheme",
    items = {
        ["center"] = {
            color = "yellow",
            ref = "drawing_train_transport10",
            contract = "train_transport10",
            item_type = "craft"
        },
    },
    arrows = {
        [0] = {
            type = "arrow_h_t_right",
            color = "yellow",
            ref = "train_impact_absorber_unlock1",
            unlock_to_hide = "drawing_train_transport10"
        },
        [1] = {
            type = "arrow_h_b_right",
            color = "yellow",
            ref = "train_automatic_couplers_unlock1",
            unlock_to_hide = "drawing_train_transport10"
        },
        [2] = {
            type = "arrow_h_c_right",
            color = "yellow",
            ref = "drawing_train_transport9",
            unlock_to_show = "drawing_train_transport10"
        }
    }
}


local cells_info_world = {
    [0] = cell_info_world_1,
    [1] = cell_info_world_2,
    [2] = cell_info_world_3,
    [3] = cell_info_world_4,
    [4] = cell_info_world_5,
    [5] = cell_info_world_6,
    [6] = cell_info_world_7,
    [7] = cell_info_world_8,
    [8] = cell_info_world_9,
    [9] = cell_info_world_10,
    [10] = cell_info_world_11,
    [11] = cell_info_world_12,
    [12] = cell_info_world_13,
    [13] = cell_info_world_14,
    [14] = cell_info_world_15   
}

local trains_type = {
	[0] = "local",
	[1] = "world"
}

local all_cells_info = {
	["local"] = cells_info_local,
	["world"] = cells_info_world,
}


local button_state = {
    [0] = "enable",
    [1] = "disable",
}

local train_type_button_colors = {
    [button_state[0]]  = "design_btn_green",
    [button_state[1]] = "design_btn_grey"
}

local is_init = false
local color_uncomplete = "grey"
local color_ready = "green"

local item_type_craft = "craft"

local current_train_type = "local"
local transport_spinboard
local train_controller

function set_train_controller( controller ) 
	train_controller = controller
end


function train_transport_design.need_update()
    if progress_widget and (progress_widget.is_force_progress_animation or progress_widget.is_animation_finished == false) then
        return false
    end

    return true
end

function change_pressed_color( button )
	local temp_image_name = button.image_name
	button.image_name = button.pressed_image
	button.pressed_image = temp_image_name
end

function change_train_type( train_type, page )

	local train_button_local = page:find_child("train_button_local", true, false)
	local train_button_world = page:find_child("train_button_world", true, false)
	if train_button_world == nil then
		errlog("[transport_spinboard::show_local_trains] Can't find button train_button_world")
		return
	end
	
	if train_button_local == nil then
		errlog("[transport_spinboard::show_local_trains] Can't find button train_button_local")
		return
	end
	
	if train_type == current_train_type then
		vrblog("[transport_spinboard::show_local_trains] Current train type alredy =", train_type)
		return
	end

    -- defult button state
	current_train_type = train_type
	
	change_pressed_color(train_button_local)
	change_pressed_color(train_button_world)
	
	local transport_spinboard = page:find_child("transport_spinboard", true, false)
	if transport_spinboard then
	    transport_spinboard.cell_width = update_spinboard_width(page)
		transport_spinboard:update_content()
		transport_spinboard:set_active_page(0)
	end

end

function show_trains(page, show)
	local train_button_local = page:find_child("train_button_local", true, false)
	if train_button_local then
		train_button_local.visible = show
		train_button_local.action = function()
									    change_train_type( "local", page  )
									end
		if is_init == false then
			train_button_local.image_name = get_d_texture_name(train_type_button_colors["enable"])
			train_button_local.pressed_image = get_d_texture_name(train_type_button_colors["disable"])--"design_btn_grey_ipad"
		end
									
	end
	
	local train_button_world = page:find_child("train_button_world", true, false)
	if train_button_world then
		train_button_world.visible = show
		train_button_world.action = function()
									    change_train_type( "world", page  )
									end
		if is_init == false then
			train_button_world.image_name = get_d_texture_name(train_type_button_colors["disable"])
			train_button_world.pressed_image = get_d_texture_name(train_type_button_colors["enable"])	
		end
	end
	
	local horizontal_separator = page:find_child("horizontal_separator", true, false)
	if horizontal_separator then
	    horizontal_separator.visible = show
	end
	
	transport_spinboard = page:find_child("transport_spinboard", true, false)
	if transport_spinboard then
		transport_spinboard.visible = show
	end
	
	if show then
		transport_spinboard.cell_width = update_spinboard_width(page)
		transport_spinboard:update_content()
		--transport_spinboard:set_active_page(0)
	end
	
	if is_init == false then
		is_init = true
	end
	
end

function update_spinboard_width(page)
    
    local cells_info = get_active_cells()--all_cells_info[current_train_type]
    local total_width = 0
    for i = 0, #cells_info do
       rect = {0, 0, 0, 0}
        local fake = page:create_child(rect)
        framework:json_ui_manager():build_from_resource(cells_info[i].type, fake)
        
        total_width = total_width + fake.rect[3]
        fake:destroy()
    end
    
    return total_width
end

function get_d_texture_name(texture_name)
	if g_device.is_tablet then
		return texture_name .. "_ipad"
	end
	return texture_name
end

function on_create_button_pressed(contract)
	vrblog("[transport_spinboard::on_create_button_pressed] button just pressed")
    train_controller:goto_factory_to_produce(contract.."_contract:create")
end


function on_make_button_pressed(unlock, parent)
	vrblog("[transport_spinboard::on_make_button_pressed] button just pressed")
    train_controller:produce_craft(unlock)
    create_draft_progress_widget(parent)
end

function on_make_button_pressed_grey(plate_descr)
	vrblog("[transport_spinboard::on_make_button_pressed_grey] button just pressed")
	local craft_item = train_controller:fill_craft( plate_descr.ref)
    if not craft_item  then
        return
    end
	           
	close_callback = 	function()
						end
		
	local params={}
	-- 0: sqc_message_box_type_default
	-- 1: sqc_message_box_type_local_error
	-- 2: sqc_message_box_type_server_error
	params.type = 0
	params.button_text = framework:strings():load("*CityOKButton","OK" )
	params.on_close_callback = close_callback
	if craft_item.missed_description == "" then
		params.dlg_text = framework:strings():load("*TransportResearchNeedComplete", "Need to complete required researches")--framework:strings():load("*TransportResearchTreeAlredyInProcess", "Another research is executing. Wait for completion")
		
	else
		params.dlg_text = craft_item.missed_description
	end
		
	framework:dialogs():show_message_box( params );
end

function fill_plate( parent, plate_to_show, plate_descr)
    
    local object_info = framework:object_factory():get_object_info_by_type_name(plate_descr.contract)
	if object_info then
	
	    -- texture
		local image = plate_to_show:find_child( "train_image" , true, false)
		if image then
		   image.texture_name = object_info.image_path
		else
		    errlog("[transport_spinboard::fill_plate] Can't find train_image widget")
		end
		
	    if plate_descr.item_type == item_type_craft then
	    	
            -- title
		    local title = plate_to_show:find_child( "title_label" , true, false)
		    if title then
		        title.caption = object_info.description
		    else
		        errlog("[transport_spinboard::fill_plate] Can't find title_label widget")  
		    end
		    
		    if current_train_type == "local" then
		        -- passengers plate
                local passengers_plate = plate_to_show:find_child( "passengers_plate" , true, false)
		        if passengers_plate then
		       		passengers_plate.visible = true
		        	local passengers_label = passengers_plate:find_child( "passengers_label" , true, false)
		        	if passengers_label then
		        		local passangers = train_controller:get_passangers_productivity(plate_descr.contract)
		        		if passangers then
		        			passengers_label.caption = string.format("+%d", passangers)
		        		end
		            end
		        end
		    else
		        -- coins plate
		        local coins_plate = plate_to_show:find_child( "coins_plate" , true, false)
		        if coins_plate then
		            coins_plate.visible = true
		            local coins_label = coins_plate:find_child( "coins_label" , true, false)
		        	if coins_label then
		        		local coins = train_controller:get_income_coins(plate_descr.contract)
		        		if coins then
		        			local compact_value = framework:formats():format_digit_compact(coins)
		        			coins_label.caption = string.format("+%s", compact_value)
		        		end
		            end
		        end
		    end

			local stops_plate = plate_to_show:find_child( "stops_plate" , true, false)
		    if stops_plate then
		        local stops_label = stops_plate:find_child( "stops_label" , true, false)
		        if stops_label then
		        	local stops = train_controller:get_max_stops(plate_descr.contract)
		        	if stops then
		        		stops_label.caption = string.format("%d", stops)
		        	end
		        end
		    end
		    
		    
		        
		    local craft_item = train_controller:fill_craft( plate_descr.ref)
		    if craft_item then
		        update_plate(plate_to_show, plate_descr, not craft_item.can_be_crafted)
		        
		        -- required
		    	local required_label = plate_to_show:find_child( "required_label" , true, false)
		    	if required_label then
		        	required_label.caption = craft_item.missed_description
		    	else
		        	errlog("[transport_spinboard::fill_plate] Can't find required_label widget")  
		    	end
		    end
		    
		    create_button(parent, plate_descr, craft_item, plate_to_show)
	    else 
	        local research_item = train_controller:get_research_by_contract_name(plate_descr.contract, plate_descr.ref)
	        if research_item  then
	            update_plate(plate_to_show, plate_descr, not research_item.check_requires)
	            
	            if research_item.research_in_progress and research_item.check_requires and train_controller:research_done(research_item) == false then
                    create_research_progress_widget(plate_to_show, research_item)
                end
            end
            
             -- level label
		    local level = plate_to_show:find_child( "level_label" , true, false)
		    if level then
			    if research_item then
			        if research_item.need_level > 1 then
			            level.caption = string.format(level.caption, research_item.current_level+1, research_item.need_level)
			        else
			            level.caption = ""
			        end
			    else
			        level.caption = ""
		        end
		    else
		        errlog("[transport_spinboard::fill_plate] Can't find level_label widget")  
		    end
        end
	else
		errlog("[transport_spinboard::fill_plate] Can't get object_info for ", plate_descr.ref)                                   
	end
end

function check_visible(idx)
    return true
end

function update_plate(plate_to_show, plate_descr, need_lock)

    if plate_descr.ref then 
        local unlock = framework:game():get_unlocker(plate_descr.ref)
        if unlock == false then
         	
         	local done_container = plate_to_show:find_child('done_container', true, false)
        	if done_container then
        	    done_container.visible = false
        	end
        	
        	if need_lock then
        	    --- image alpha
                local item_image = plate_to_show:find_child('train_image', true, false)
                if item_image then
            	    local grayscaled = plate_to_show:find_child('grayscaled', true, false)
        	        if grayscaled then 
        		        item_image:set_parent(grayscaled)
        		        item_image.alpha = 0.5
        	        end
        	
        	        local lock_image = plate_to_show:find_child('lock_image', true, false)
        	        if lock_image then
        	            lock_image.visible = true
        	        end
                end
        	end 
        end
    else
        local done_container = plate_to_show:find_child('done_container', true, false)
        if done_container then
        	done_container.visible = false
        end
    end
end

function get_active_cells()
    local idx = 0
    local active_cells = {}
    for i =0, #all_cells_info[current_train_type] do
        local c_info = all_cells_info[current_train_type][i]
        if c_info.unlock_to_hide then
            local unlock = framework:game():get_unlocker(c_info.unlock_to_hide)
            if unlock == false then
                active_cells[idx] = c_info
                idx = idx + 1
            end
        else 
            active_cells[idx] = c_info
            idx = idx + 1
        end
    end
    
    return active_cells
end

function get_active_arrows(arrows)
    local idx = 0
    local active_arrows = {}
    for i =0, #arrows do
        local arrow = arrows[i]
        if arrow.unlock_to_hide then
            local unlock = framework:game():get_unlocker(arrow.unlock_to_hide)
            if unlock == false then
                active_arrows[idx] = arrow
                idx = idx + 1
            end
        elseif arrow.unlock_to_show then
            local unlock = framework:game():get_unlocker(arrow.unlock_to_show)
            if unlock == true then
                active_arrows[idx] = arrow
                idx = idx + 1
            end
        else 
            active_arrows[idx] = arrow
            idx = idx + 1
        end
    end
    
    return active_arrows
end

function force_research(research_item)
    if progress_widget then
        if progress_widget.is_force_progress_animation then
            return
        end
        
        progress_widget:force_progress()
    end
    
    train_controller:force_research(research_item)
end

function create_draft_progress_widget(parent)
    progress_widget = create_progress_widget(parent, "train_draft_progress_bg", "train_draft_progress_mask")
    if not progress_widget then return end

    progress_widget:start_animation()
end

function create_research_progress_widget(parent, research_item)
    progress_widget = create_progress_widget(parent, "blind", "research_progress_mask")
    if not progress_widget then return end

    local current_progress_time = train_controller:research_in_progress_time(research_item)
    local all_progress_time = train_controller:research_length_time(research_item) * 1000
    progress_widget.current_progress_time = all_progress_time - current_progress_time
    progress_widget.all_progress_time = all_progress_time

    progress_widget:start_animation()
end

function create_progress_widget(parent, background, mask)
    local progress_widget = parent:find_child("progress_widget", true, false)
    if not progress_widget then return nil end

    progress_widget.background_image =  background
    progress_widget.sectors_image = mask
        
    progress_mask = parent:find_child("progress_mask", true, false)
    if progress_mask then
        local progress_animation_size = {progress_mask.rect[3], progress_mask.rect[4]}
        local progress_animation_offset = {progress_mask.rect[1], progress_mask.rect[2]}
            
        progress_widget.image_size = progress_animation_size
        progress_widget.progress_offset = progress_animation_offset
    end

    progress_widget.finish_callback = function()
        transport_spinboard:update_content()
    end

    return progress_widget
end

function on_plate_pessed(parent, research_item, bubble_rect)
    if not research_item then return end

    if bubble then
        bubble:on_destroy()
        bubble = {}
    end

    bubble = create_research_bubble(research_item, train_controller, parent)

    if not bubble then return end

    bubble:init()
    
    local current_rect = bubble:get_rect()
    current_rect[1] = bubble_rect[1]
    current_rect[2] = bubble_rect[2] + bubble_rect[4] - current_rect[4] 
    bubble:set_rect(current_rect)
    
    bubble.start_callback = function()
                                transport_spinboard:update_content()
                            end

    bubble.force_callback = function()
                                force_research(research_item)
                            end
end
function create_button(cell, plate_descr, craft_item, item_plate)

	local unlocker = framework:game():get_unlocker(plate_descr.ref)
	if plate_descr.ref == nil or unlocker == true then
		local create_button = cell:find_child( "create_button" , true, false)
    	if create_button then
       		create_button.visible = true
       		create_button.active = true
       		create_button.action = 	function()
                                  		on_create_button_pressed(plate_descr.contract)
                              		end
    	end
    else
        if craft_item and craft_item.can_be_crafted then
    	    local make_button = cell:find_child( "make_button" , true, false)
    	    if make_button then
       		    make_button.visible = true
       		    make_button.active = true
       		    make_button.action = 	function()
                                  		    on_make_button_pressed(plate_descr.ref, item_plate)
                              		    end
            end
        else
            local make_button = cell:find_child( "make_button_grey" , true, false)
    	    if make_button then
       		    make_button.visible = true
       		    make_button.active = true
       		    make_button.action = 	function()
                                  		    on_make_button_pressed_grey(plate_descr)
                              		    end
            end
        end
	end
	
end

function check_special_plate_color(plate_color, plate_descr)

    local research_item = train_controller:get_research_by_contract_name(plate_descr.contract, plate_descr.ref)
	local craft_item = train_controller:fill_craft( plate_descr.ref)
    if plate_descr.ref then
		local unlock = framework:game():get_unlocker(plate_descr.ref)
        if unlock == false then
             plate_color = color_uncomplete
        end
                        
        if research_item then 
            if train_controller:research_done(research_item) == false and research_item.check_requires == true then
                plate_color = color_ready
            end
        elseif craft_item then
            if craft_item.can_be_crafted == true then
                plate_color = color_ready
            end
        end
	end
	
	return plate_color
end

function check_arrow_color( color, arrow_info )
    if arrow_info.ref then
	    local unlock = framework:game():get_unlocker(arrow_info.ref)
        if unlock == true then
            color = arrow_info.color
        end
	else
		color = arrow_info.color
	end 
	return color
end

function show_bubble( parent, w, plate_to_show, plate_descr)
     if progress_widget and progress_widget.is_force_progress_animation then
        return
     end
     
     local research_item = train_controller:get_research_by_contract_name(plate_descr.contract, plate_descr.ref)

     local bubble_rect = plate_to_show.rect
     bubble_rect[1] = w.rect[1] + bubble_rect[1] + bubble_rect[3]
     bubble_rect[2] = bubble_rect[2]
     on_plate_pessed(parent, research_item, bubble_rect)
end

function create_pseudo_cell( parent, idx, rect)

        check_visible(idx)
        local cells_info = get_active_cells()--all_cells_info[current_train_type]
        local c_info = cells_info[idx]
        local w = parent:create_child(rect)
        
		framework:json_ui_manager():build_from_resource(c_info.type,w)
		local cur_cell_width = w.rect[3]
		rect[3] = cur_cell_width
		w.rect = rect
		--- PLATES
		for i = 0, #plate_positions do
		    local position = plate_positions[i]
		    if position then
		        local plate_descr = c_info.items[position]
		        if plate_descr then
		            plate_color = plate_descr.color
		            plate_color = check_special_plate_color(plate_color, plate_descr)
		            if plate_color then
		                local plates_textures_by_color = plate_color_names[plate_color]
		                if plates_textures_by_color then
		                    local type = c_info.type
		                    if type then 
		                        local plate_texture_name = plates_textures_by_color[type]
		                        if plate_texture_name then
		                            local plate_to_show = w:find_child( "plate_" .. position, true, false)
		                            if plate_to_show then 
                                        plate_to_show.visible = true
                                        plate_to_show.image_name = get_d_texture_name(plate_texture_name)

                                        plate_to_show.action =  function()
                                                                    show_bubble( parent, w, plate_to_show, plate_descr)
                                                                end
                                        
                                        fill_plate(w, plate_to_show, plate_descr)
                                    end
                                else
		                            errlog("[transport_spinboard::create_cell] There is no texture for plates type =", plate_descr.type, "and color =", plate_color)
		                        end		  
		                    else
		                        errlog("[transport_spinboard::create_cell] There is no plates with color ", plate_color, "and type = ", type)
		                    end         
		                else
		                    errlog("[transport_spinboard::create_cell] There is no plates with color ", plate_color)
		                end
		            else
		                errlog ("[transport_spinboard::create_cell] No color for plate with position", position," for cell with idx = ", idx) 
		            end 
		        else
		            --print ("[transport_spinboard::create_cell] No plate with position", position," for cell with idx = ", idx) 
		        end
		    else
		        errlog ("[transport_spinboard::create_cell] Wrong position for plate with index", i)
		    end
		end
		
		--- ARROWS
		local arrows = get_active_arrows(c_info.arrows)
		for i =0 , #arrows do 
		    local arrow_info = arrows[i]
		    if arrow_info then
		        local color = color_uncomplete
		        color = check_arrow_color(color, arrow_info)
		        --check_arrow_color(color, arrow_info)
		        if color then
		            local color_map = arrow_color_names[color]
		            if color_map then
		                local arrow_type = arrow_info.type
		                if arrow_type then
		                    local arrow_texture = color_map[arrow_type]
		                    if arrow_texture then
		                        local arrow_to_show = w:find_child(arrow_type, true, false)
		                        if arrow_to_show then
		                            arrow_to_show.image_name = get_d_texture_name(arrow_texture)
		                            arrow_to_show.visible = true
		                        else
		                            errlog ("[transport_spinboard::create_cell] Can't find widget with name", arrow_type)
		                        end
		                    else
		                        errlog ("[transport_spinboard::create_cell] Can't find texture for color", color, ", arrow type", a_type)
		                    end
		                else
		                    errlog ("[transport_spinboard::create_cell] Wrong arrow type for arrow with index", i, "on cell with idx", idx)
		                end
		            else
		                errlog ("[transport_spinboard::create_cell] Can't find color", color, "for arrow with index", i ,"on cell with idx", idx)
		            end
		        else
		            errlog ("[transport_spinboard::create_cell] No color for arrow with index", i, "on cell with idx", idx)
		        end
		    else
		        errlog ("[transport_spinboard::create_cell] There is no arrow on cell with idx=", idx)
		    end
		end
		
		rect[1] = rect[1] + cur_cell_width
end

transport_spinboard_delegate = {
    create_cell = function (  self, idx, rect )

        local w = self.spinboard:create_child(rect)
        w.rect = rect

		local cells_info = get_active_cells()--all_cells_info[current_train_type]
        for j = 0, #cells_info do
            create_pseudo_cell( w, j, rect)
        end

		return w
    end,
    
    get_cell_count = function ( )
		return  1
    end
}


return train_transport_design
